# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

# This wrapper is part of IMP,
# Copyright 2007-2021 IMP Inventors. All rights reserved.

from __future__ import print_function, division, absolute_import
from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

import _IMP_misc

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

class IMP_MISC_SwigPyIterator(object):
    r"""Proxy of C++ swig::IMP_MISC_SwigPyIterator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _IMP_misc.delete_IMP_MISC_SwigPyIterator

    def value(self):
        r"""value(IMP_MISC_SwigPyIterator self) -> PyObject *"""
        return _IMP_misc.IMP_MISC_SwigPyIterator_value(self)

    def incr(self, n=1):
        r"""incr(IMP_MISC_SwigPyIterator self, size_t n=1) -> IMP_MISC_SwigPyIterator"""
        return _IMP_misc.IMP_MISC_SwigPyIterator_incr(self, n)

    def decr(self, n=1):
        r"""decr(IMP_MISC_SwigPyIterator self, size_t n=1) -> IMP_MISC_SwigPyIterator"""
        return _IMP_misc.IMP_MISC_SwigPyIterator_decr(self, n)

    def distance(self, x):
        r"""distance(IMP_MISC_SwigPyIterator self, IMP_MISC_SwigPyIterator x) -> ptrdiff_t"""
        return _IMP_misc.IMP_MISC_SwigPyIterator_distance(self, x)

    def equal(self, x):
        r"""equal(IMP_MISC_SwigPyIterator self, IMP_MISC_SwigPyIterator x) -> bool"""
        return _IMP_misc.IMP_MISC_SwigPyIterator_equal(self, x)

    def copy(self):
        r"""copy(IMP_MISC_SwigPyIterator self) -> IMP_MISC_SwigPyIterator"""
        return _IMP_misc.IMP_MISC_SwigPyIterator_copy(self)

    def next(self):
        r"""next(IMP_MISC_SwigPyIterator self) -> PyObject *"""
        return _IMP_misc.IMP_MISC_SwigPyIterator_next(self)

    def __next__(self):
        r"""__next__(IMP_MISC_SwigPyIterator self) -> PyObject *"""
        return _IMP_misc.IMP_MISC_SwigPyIterator___next__(self)

    def previous(self):
        r"""previous(IMP_MISC_SwigPyIterator self) -> PyObject *"""
        return _IMP_misc.IMP_MISC_SwigPyIterator_previous(self)

    def advance(self, n):
        r"""advance(IMP_MISC_SwigPyIterator self, ptrdiff_t n) -> IMP_MISC_SwigPyIterator"""
        return _IMP_misc.IMP_MISC_SwigPyIterator_advance(self, n)

    def __eq__(self, x):
        r"""__eq__(IMP_MISC_SwigPyIterator self, IMP_MISC_SwigPyIterator x) -> bool"""
        return _IMP_misc.IMP_MISC_SwigPyIterator___eq__(self, x)

    def __ne__(self, x):
        r"""__ne__(IMP_MISC_SwigPyIterator self, IMP_MISC_SwigPyIterator x) -> bool"""
        return _IMP_misc.IMP_MISC_SwigPyIterator___ne__(self, x)

    def __iadd__(self, n):
        r"""__iadd__(IMP_MISC_SwigPyIterator self, ptrdiff_t n) -> IMP_MISC_SwigPyIterator"""
        return _IMP_misc.IMP_MISC_SwigPyIterator___iadd__(self, n)

    def __isub__(self, n):
        r"""__isub__(IMP_MISC_SwigPyIterator self, ptrdiff_t n) -> IMP_MISC_SwigPyIterator"""
        return _IMP_misc.IMP_MISC_SwigPyIterator___isub__(self, n)

    def __add__(self, n):
        r"""__add__(IMP_MISC_SwigPyIterator self, ptrdiff_t n) -> IMP_MISC_SwigPyIterator"""
        return _IMP_misc.IMP_MISC_SwigPyIterator___add__(self, n)

    def __sub__(self, *args):
        r"""
        __sub__(IMP_MISC_SwigPyIterator self, ptrdiff_t n) -> IMP_MISC_SwigPyIterator
        __sub__(IMP_MISC_SwigPyIterator self, IMP_MISC_SwigPyIterator x) -> ptrdiff_t
        """
        return _IMP_misc.IMP_MISC_SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register IMP_MISC_SwigPyIterator in _IMP_misc:
_IMP_misc.IMP_MISC_SwigPyIterator_swigregister(IMP_MISC_SwigPyIterator)


_value_types=[]
_object_types=[]
_raii_types=[]
_plural_types=[]

IMP_DEBUG = _IMP_misc.IMP_DEBUG

IMP_RELEASE = _IMP_misc.IMP_RELEASE

IMP_SILENT = _IMP_misc.IMP_SILENT

IMP_PROGRESS = _IMP_misc.IMP_PROGRESS

IMP_TERSE = _IMP_misc.IMP_TERSE

IMP_VERBOSE = _IMP_misc.IMP_VERBOSE

IMP_MEMORY = _IMP_misc.IMP_MEMORY

IMP_NONE = _IMP_misc.IMP_NONE

IMP_USAGE = _IMP_misc.IMP_USAGE

IMP_INTERNAL = _IMP_misc.IMP_INTERNAL

IMP_KERNEL_HAS_LOG4CXX = _IMP_misc.IMP_KERNEL_HAS_LOG4CXX

IMP_COMPILER_HAS_AUTO = _IMP_misc.IMP_COMPILER_HAS_AUTO

IMP_COMPILER_HAS_DEBUG_VECTOR = _IMP_misc.IMP_COMPILER_HAS_DEBUG_VECTOR

IMP_COMPILER_HAS_UNIQUE_PTR = _IMP_misc.IMP_COMPILER_HAS_UNIQUE_PTR

IMP_KERNEL_HAS_BOOST_RANDOM = _IMP_misc.IMP_KERNEL_HAS_BOOST_RANDOM

IMP_KERNEL_HAS_NUMPY = _IMP_misc.IMP_KERNEL_HAS_NUMPY

IMP_KERNEL_HAS_GPERFTOOLS = _IMP_misc.IMP_KERNEL_HAS_GPERFTOOLS

IMP_KERNEL_HAS_TCMALLOC_HEAPCHECKER = _IMP_misc.IMP_KERNEL_HAS_TCMALLOC_HEAPCHECKER

IMP_KERNEL_HAS_TCMALLOC_HEAPPROFILER = _IMP_misc.IMP_KERNEL_HAS_TCMALLOC_HEAPPROFILER

IMPKERNEL_SHOW_WARNINGS = _IMP_misc.IMPKERNEL_SHOW_WARNINGS


import sys
class _DirectorObjects(object):
    """@internal Simple class to keep references to director objects
       to prevent premature deletion."""
    def __init__(self):
        self._objects = []
    def register(self, obj):
        """Take a reference to a director object; will only work for
           refcounted C++ classes"""
        if hasattr(obj, 'get_ref_count'):
            self._objects.append(obj)
    def cleanup(self):
        """Only drop our reference and allow cleanup by Python if no other
           Python references exist (we hold 3 references: one in self._objects,
           one in x, and one in the argument list for getrefcount) *and* no
           other C++ references exist (the Python object always holds one)"""
        objs = [x for x in self._objects if sys.getrefcount(x) > 3 \
                                         or x.get_ref_count() > 1]
# Do in two steps so the references are kept until the end of the
# function (deleting references may trigger a fresh call to this method)
        self._objects = objs
    def get_object_count(self):
        """Get number of director objects (useful for testing only)"""
        return len(self._objects)
_director_objects = _DirectorObjects()

class _ostream(object):
    r"""Proxy of C++ std::ostream class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def write(self, osa_buf):
        r"""write(_ostream self, char const * osa_buf)"""
        return _IMP_misc._ostream_write(self, osa_buf)

# Register _ostream in _IMP_misc:
_IMP_misc._ostream_swigregister(_ostream)

IMP_COMPILER_HAS_OVERRIDE = _IMP_misc.IMP_COMPILER_HAS_OVERRIDE

IMP_COMPILER_HAS_FINAL = _IMP_misc.IMP_COMPILER_HAS_FINAL

IMP_HAS_NOEXCEPT = _IMP_misc.IMP_HAS_NOEXCEPT

IMP_C_OPEN_BINARY = _IMP_misc.IMP_C_OPEN_BINARY

import IMP
IMP_CGAL_HAS_BOOST_FILESYSTEM = _IMP_misc.IMP_CGAL_HAS_BOOST_FILESYSTEM

IMP_CGAL_HAS_BOOST_PROGRAMOPTIONS = _IMP_misc.IMP_CGAL_HAS_BOOST_PROGRAMOPTIONS

IMP_CGAL_HAS_BOOST_RANDOM = _IMP_misc.IMP_CGAL_HAS_BOOST_RANDOM

IMP_CGAL_HAS_BOOST_SYSTEM = _IMP_misc.IMP_CGAL_HAS_BOOST_SYSTEM

IMP_CGAL_HAS_NUMPY = _IMP_misc.IMP_CGAL_HAS_NUMPY

IMPCGAL_SHOW_WARNINGS = _IMP_misc.IMPCGAL_SHOW_WARNINGS

import IMP.cgal
IMP_ALGEBRA_HAS_IMP_CGAL = _IMP_misc.IMP_ALGEBRA_HAS_IMP_CGAL

IMP_ALGEBRA_HAS_BOOST_FILESYSTEM = _IMP_misc.IMP_ALGEBRA_HAS_BOOST_FILESYSTEM

IMP_ALGEBRA_HAS_BOOST_PROGRAMOPTIONS = _IMP_misc.IMP_ALGEBRA_HAS_BOOST_PROGRAMOPTIONS

IMP_ALGEBRA_HAS_BOOST_RANDOM = _IMP_misc.IMP_ALGEBRA_HAS_BOOST_RANDOM

IMP_ALGEBRA_HAS_BOOST_SYSTEM = _IMP_misc.IMP_ALGEBRA_HAS_BOOST_SYSTEM

IMP_ALGEBRA_HAS_CGAL = _IMP_misc.IMP_ALGEBRA_HAS_CGAL

IMP_ALGEBRA_HAS_NUMPY = _IMP_misc.IMP_ALGEBRA_HAS_NUMPY

IMP_ALGEBRA_HAS_ANN = _IMP_misc.IMP_ALGEBRA_HAS_ANN

IMPALGEBRA_SHOW_WARNINGS = _IMP_misc.IMPALGEBRA_SHOW_WARNINGS

import IMP.algebra
IMP_DISPLAY_HAS_IMP_CGAL = _IMP_misc.IMP_DISPLAY_HAS_IMP_CGAL

IMP_DISPLAY_HAS_BOOST_FILESYSTEM = _IMP_misc.IMP_DISPLAY_HAS_BOOST_FILESYSTEM

IMP_DISPLAY_HAS_BOOST_PROGRAMOPTIONS = _IMP_misc.IMP_DISPLAY_HAS_BOOST_PROGRAMOPTIONS

IMP_DISPLAY_HAS_BOOST_RANDOM = _IMP_misc.IMP_DISPLAY_HAS_BOOST_RANDOM

IMP_DISPLAY_HAS_BOOST_SYSTEM = _IMP_misc.IMP_DISPLAY_HAS_BOOST_SYSTEM

IMP_DISPLAY_HAS_CGAL = _IMP_misc.IMP_DISPLAY_HAS_CGAL

IMP_DISPLAY_HAS_NUMPY = _IMP_misc.IMP_DISPLAY_HAS_NUMPY

IMPDISPLAY_SHOW_WARNINGS = _IMP_misc.IMPDISPLAY_SHOW_WARNINGS

import IMP.display
IMP_SCORE_FUNCTOR_HAS_IMP_CGAL = _IMP_misc.IMP_SCORE_FUNCTOR_HAS_IMP_CGAL

IMP_SCORE_FUNCTOR_HAS_BOOST_FILESYSTEM = _IMP_misc.IMP_SCORE_FUNCTOR_HAS_BOOST_FILESYSTEM

IMP_SCORE_FUNCTOR_HAS_BOOST_PROGRAMOPTIONS = _IMP_misc.IMP_SCORE_FUNCTOR_HAS_BOOST_PROGRAMOPTIONS

IMP_SCORE_FUNCTOR_HAS_BOOST_RANDOM = _IMP_misc.IMP_SCORE_FUNCTOR_HAS_BOOST_RANDOM

IMP_SCORE_FUNCTOR_HAS_BOOST_SYSTEM = _IMP_misc.IMP_SCORE_FUNCTOR_HAS_BOOST_SYSTEM

IMP_SCORE_FUNCTOR_HAS_CGAL = _IMP_misc.IMP_SCORE_FUNCTOR_HAS_CGAL

IMP_SCORE_FUNCTOR_HAS_HDF5 = _IMP_misc.IMP_SCORE_FUNCTOR_HAS_HDF5

IMP_SCORE_FUNCTOR_HAS_NUMPY = _IMP_misc.IMP_SCORE_FUNCTOR_HAS_NUMPY

IMPSCOREFUNCTOR_SHOW_WARNINGS = _IMP_misc.IMPSCOREFUNCTOR_SHOW_WARNINGS

import IMP.score_functor
IMP_CORE_HAS_IMP_CGAL = _IMP_misc.IMP_CORE_HAS_IMP_CGAL

IMP_CORE_HAS_IMP_KERNEL = _IMP_misc.IMP_CORE_HAS_IMP_KERNEL

IMP_CORE_HAS_BOOST_FILESYSTEM = _IMP_misc.IMP_CORE_HAS_BOOST_FILESYSTEM

IMP_CORE_HAS_BOOST_PROGRAMOPTIONS = _IMP_misc.IMP_CORE_HAS_BOOST_PROGRAMOPTIONS

IMP_CORE_HAS_BOOST_RANDOM = _IMP_misc.IMP_CORE_HAS_BOOST_RANDOM

IMP_CORE_HAS_BOOST_SYSTEM = _IMP_misc.IMP_CORE_HAS_BOOST_SYSTEM

IMP_CORE_HAS_CGAL = _IMP_misc.IMP_CORE_HAS_CGAL

IMP_CORE_HAS_HDF5 = _IMP_misc.IMP_CORE_HAS_HDF5

IMP_CORE_HAS_NUMPY = _IMP_misc.IMP_CORE_HAS_NUMPY

IMPCORE_SHOW_WARNINGS = _IMP_misc.IMPCORE_SHOW_WARNINGS

import IMP.core
IMP_CONTAINER_HAS_IMP_ALGEBRA = _IMP_misc.IMP_CONTAINER_HAS_IMP_ALGEBRA

IMP_CONTAINER_HAS_IMP_CGAL = _IMP_misc.IMP_CONTAINER_HAS_IMP_CGAL

IMP_CONTAINER_HAS_IMP_DISPLAY = _IMP_misc.IMP_CONTAINER_HAS_IMP_DISPLAY

IMP_CONTAINER_HAS_IMP_KERNEL = _IMP_misc.IMP_CONTAINER_HAS_IMP_KERNEL

IMP_CONTAINER_HAS_IMP_SCORE_FUNCTOR = _IMP_misc.IMP_CONTAINER_HAS_IMP_SCORE_FUNCTOR

IMP_CONTAINER_HAS_BOOST_FILESYSTEM = _IMP_misc.IMP_CONTAINER_HAS_BOOST_FILESYSTEM

IMP_CONTAINER_HAS_BOOST_PROGRAMOPTIONS = _IMP_misc.IMP_CONTAINER_HAS_BOOST_PROGRAMOPTIONS

IMP_CONTAINER_HAS_BOOST_RANDOM = _IMP_misc.IMP_CONTAINER_HAS_BOOST_RANDOM

IMP_CONTAINER_HAS_BOOST_SYSTEM = _IMP_misc.IMP_CONTAINER_HAS_BOOST_SYSTEM

IMP_CONTAINER_HAS_CGAL = _IMP_misc.IMP_CONTAINER_HAS_CGAL

IMP_CONTAINER_HAS_HDF5 = _IMP_misc.IMP_CONTAINER_HAS_HDF5

IMP_CONTAINER_HAS_NUMPY = _IMP_misc.IMP_CONTAINER_HAS_NUMPY

IMP_CONTAINER_HAS_PYTHON_IHM = _IMP_misc.IMP_CONTAINER_HAS_PYTHON_IHM

IMP_CONTAINER_HAS_GOOGLE_DENSE_HASH_MAP = _IMP_misc.IMP_CONTAINER_HAS_GOOGLE_DENSE_HASH_MAP

IMP_CONTAINER_HAS_ROBIN_MAP = _IMP_misc.IMP_CONTAINER_HAS_ROBIN_MAP

IMPCONTAINER_SHOW_WARNINGS = _IMP_misc.IMPCONTAINER_SHOW_WARNINGS

import IMP.container
IMP_ATOM_HAS_IMP_CGAL = _IMP_misc.IMP_ATOM_HAS_IMP_CGAL

IMP_ATOM_HAS_IMP_KERNEL = _IMP_misc.IMP_ATOM_HAS_IMP_KERNEL

IMP_ATOM_HAS_IMP_SCORE_FUNCTOR = _IMP_misc.IMP_ATOM_HAS_IMP_SCORE_FUNCTOR

IMP_ATOM_HAS_BOOST_PROGRAMOPTIONS = _IMP_misc.IMP_ATOM_HAS_BOOST_PROGRAMOPTIONS

IMP_ATOM_HAS_BOOST_RANDOM = _IMP_misc.IMP_ATOM_HAS_BOOST_RANDOM

IMP_ATOM_HAS_BOOST_REGEX = _IMP_misc.IMP_ATOM_HAS_BOOST_REGEX

IMP_ATOM_HAS_BOOST_SYSTEM = _IMP_misc.IMP_ATOM_HAS_BOOST_SYSTEM

IMP_ATOM_HAS_CGAL = _IMP_misc.IMP_ATOM_HAS_CGAL

IMP_ATOM_HAS_HDF5 = _IMP_misc.IMP_ATOM_HAS_HDF5

IMP_ATOM_HAS_NUMPY = _IMP_misc.IMP_ATOM_HAS_NUMPY

IMP_ATOM_HAS_PYTHON_IHM = _IMP_misc.IMP_ATOM_HAS_PYTHON_IHM

IMPATOM_SHOW_WARNINGS = _IMP_misc.IMPATOM_SHOW_WARNINGS

IMP_ATOM_TYPE_INDEX = _IMP_misc.IMP_ATOM_TYPE_INDEX

IMP_RESIDUE_TYPE_INDEX = _IMP_misc.IMP_RESIDUE_TYPE_INDEX

IMP_HIERARCHY_TYPE_INDEX = _IMP_misc.IMP_HIERARCHY_TYPE_INDEX

IMP_CHAIN_TYPE_INDEX = _IMP_misc.IMP_CHAIN_TYPE_INDEX

import IMP.atom
IMP_MISC_HAS_IMP_ALGEBRA = _IMP_misc.IMP_MISC_HAS_IMP_ALGEBRA

IMP_MISC_HAS_IMP_CGAL = _IMP_misc.IMP_MISC_HAS_IMP_CGAL

IMP_MISC_HAS_IMP_CONTAINER = _IMP_misc.IMP_MISC_HAS_IMP_CONTAINER

IMP_MISC_HAS_IMP_DISPLAY = _IMP_misc.IMP_MISC_HAS_IMP_DISPLAY

IMP_MISC_HAS_IMP_KERNEL = _IMP_misc.IMP_MISC_HAS_IMP_KERNEL

IMP_MISC_HAS_IMP_SCORE_FUNCTOR = _IMP_misc.IMP_MISC_HAS_IMP_SCORE_FUNCTOR

IMP_MISC_HAS_BOOST_FILESYSTEM = _IMP_misc.IMP_MISC_HAS_BOOST_FILESYSTEM

IMP_MISC_HAS_BOOST_PROGRAMOPTIONS = _IMP_misc.IMP_MISC_HAS_BOOST_PROGRAMOPTIONS

IMP_MISC_HAS_BOOST_RANDOM = _IMP_misc.IMP_MISC_HAS_BOOST_RANDOM

IMP_MISC_HAS_BOOST_REGEX = _IMP_misc.IMP_MISC_HAS_BOOST_REGEX

IMP_MISC_HAS_BOOST_SYSTEM = _IMP_misc.IMP_MISC_HAS_BOOST_SYSTEM

IMP_MISC_HAS_CGAL = _IMP_misc.IMP_MISC_HAS_CGAL

IMP_MISC_HAS_HDF5 = _IMP_misc.IMP_MISC_HAS_HDF5

IMP_MISC_HAS_NUMPY = _IMP_misc.IMP_MISC_HAS_NUMPY

IMP_MISC_HAS_PYTHON_IHM = _IMP_misc.IMP_MISC_HAS_PYTHON_IHM

IMPMISC_SHOW_WARNINGS = _IMP_misc.IMPMISC_SHOW_WARNINGS


_object_types.append("WormLikeChain")


def _object_cast_to_WormLikeChain(o):
    r"""_object_cast_to_WormLikeChain(Object o) -> WormLikeChain"""
    return _IMP_misc._object_cast_to_WormLikeChain(o)

_object_types.append("FreelyJointedChain")


def _object_cast_to_FreelyJointedChain(o):
    r"""_object_cast_to_FreelyJointedChain(Object o) -> FreelyJointedChain"""
    return _IMP_misc._object_cast_to_FreelyJointedChain(o)

_object_types.append("StateAdaptor")


def _object_cast_to_StateAdaptor(o):
    r"""_object_cast_to_StateAdaptor(Object o) -> StateAdaptor"""
    return _IMP_misc._object_cast_to_StateAdaptor(o)

_object_types.append("CommonEndpointPairFilter")


def _object_cast_to_CommonEndpointPairFilter(o):
    r"""_object_cast_to_CommonEndpointPairFilter(Object o) -> CommonEndpointPairFilter"""
    return _IMP_misc._object_cast_to_CommonEndpointPairFilter(o)

_object_types.append("LogPairScore")


def _object_cast_to_LogPairScore(o):
    r"""_object_cast_to_LogPairScore(Object o) -> LogPairScore"""
    return _IMP_misc._object_cast_to_LogPairScore(o)

_object_types.append("LowestRefinedPairScore")


def _object_cast_to_LowestRefinedPairScore(o):
    r"""_object_cast_to_LowestRefinedPairScore(Object o) -> LowestRefinedPairScore"""
    return _IMP_misc._object_cast_to_LowestRefinedPairScore(o)

_object_types.append("SoftCylinderPairScore")


def _object_cast_to_SoftCylinderPairScore(o):
    r"""_object_cast_to_SoftCylinderPairScore(Object o) -> SoftCylinderPairScore"""
    return _IMP_misc._object_cast_to_SoftCylinderPairScore(o)

_object_types.append("DecayPairContainerOptimizerState")


def _object_cast_to_DecayPairContainerOptimizerState(o):
    r"""_object_cast_to_DecayPairContainerOptimizerState(Object o) -> DecayPairContainerOptimizerState"""
    return _IMP_misc._object_cast_to_DecayPairContainerOptimizerState(o)

def CustomXYZRs(l=[]):
    return [CustomXYZR(x) for x in l]
_plural_types.append("CustomXYZRs")


_value_types.append("CustomXYZR")

class WormLikeChain(IMP.UnaryFunction):
    r"""Proxy of C++ IMP::misc::WormLikeChain class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, l_max, lp):
        r"""__init__(WormLikeChain self, IMP::Float l_max, IMP::Float lp) -> WormLikeChain"""
        _IMP_misc.WormLikeChain_swiginit(self, _IMP_misc.new_WormLikeChain(l_max, lp))

    def get_version_info(self):
        r"""get_version_info(WormLikeChain self) -> VersionInfo"""
        return _IMP_misc.WormLikeChain_get_version_info(self)

    def do_show(self, out):
        r"""do_show(WormLikeChain self, _ostream out)"""
        return _IMP_misc.WormLikeChain_do_show(self, out)

    def __str__(self):
        r"""__str__(WormLikeChain self) -> std::string"""
        return _IMP_misc.WormLikeChain___str__(self)

    def __repr__(self):
        r"""__repr__(WormLikeChain self) -> std::string"""
        return _IMP_misc.WormLikeChain___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_WormLikeChain(o)


# Register WormLikeChain in _IMP_misc:
_IMP_misc.WormLikeChain_swigregister(WormLikeChain)

class FreelyJointedChain(IMP.UnaryFunction):
    r"""Proxy of C++ IMP::misc::FreelyJointedChain class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, link_num, link_length):
        r"""__init__(FreelyJointedChain self, int link_num, double link_length) -> FreelyJointedChain"""
        _IMP_misc.FreelyJointedChain_swiginit(self, _IMP_misc.new_FreelyJointedChain(link_num, link_length))

    def get_version_info(self):
        r"""get_version_info(FreelyJointedChain self) -> VersionInfo"""
        return _IMP_misc.FreelyJointedChain_get_version_info(self)

    def get_link_number(self):
        r"""get_link_number(FreelyJointedChain self) -> int"""
        return _IMP_misc.FreelyJointedChain_get_link_number(self)

    def get_link_length(self):
        r"""get_link_length(FreelyJointedChain self) -> double"""
        return _IMP_misc.FreelyJointedChain_get_link_length(self)

    def set_link_number(self, N):
        r"""set_link_number(FreelyJointedChain self, int N)"""
        return _IMP_misc.FreelyJointedChain_set_link_number(self, N)

    def set_link_length(self, b):
        r"""set_link_length(FreelyJointedChain self, double b)"""
        return _IMP_misc.FreelyJointedChain_set_link_length(self, b)

    def get_distance_at_minimum(self):
        r"""get_distance_at_minimum(FreelyJointedChain self) -> double"""
        return _IMP_misc.FreelyJointedChain_get_distance_at_minimum(self)

    def get_average_distance(self):
        r"""get_average_distance(FreelyJointedChain self) -> double"""
        return _IMP_misc.FreelyJointedChain_get_average_distance(self)

    def __str__(self):
        r"""__str__(FreelyJointedChain self) -> std::string"""
        return _IMP_misc.FreelyJointedChain___str__(self)

    def __repr__(self):
        r"""__repr__(FreelyJointedChain self) -> std::string"""
        return _IMP_misc.FreelyJointedChain___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_FreelyJointedChain(o)


# Register FreelyJointedChain in _IMP_misc:
_IMP_misc.FreelyJointedChain_swigregister(FreelyJointedChain)

class LowestRefinedPairScore(IMP.PairScore):
    r"""Proxy of C++ IMP::misc::LowestRefinedPairScore class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, r, f):
        r"""__init__(LowestRefinedPairScore self, Refiner r, PairScore f) -> LowestRefinedPairScore"""
        _IMP_misc.LowestRefinedPairScore_swiginit(self, _IMP_misc.new_LowestRefinedPairScore(r, f))

    def do_get_inputs(self, m, pis):
        r"""do_get_inputs(LowestRefinedPairScore self, Model m, IMP::ParticleIndexes const & pis) -> IMP::ModelObjectsTemp"""
        return _IMP_misc.LowestRefinedPairScore_do_get_inputs(self, m, pis)

    def get_version_info(self):
        r"""get_version_info(LowestRefinedPairScore self) -> VersionInfo"""
        return _IMP_misc.LowestRefinedPairScore_get_version_info(self)

    def __str__(self):
        r"""__str__(LowestRefinedPairScore self) -> std::string"""
        return _IMP_misc.LowestRefinedPairScore___str__(self)

    def __repr__(self):
        r"""__repr__(LowestRefinedPairScore self) -> std::string"""
        return _IMP_misc.LowestRefinedPairScore___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_LowestRefinedPairScore(o)


# Register LowestRefinedPairScore in _IMP_misc:
_IMP_misc.LowestRefinedPairScore_swigregister(LowestRefinedPairScore)

class LogPairScore(IMP.PairScore):
    r"""Proxy of C++ IMP::misc::LogPairScore class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self):
        r"""__init__(LogPairScore self) -> LogPairScore"""
        _IMP_misc.LogPairScore_swiginit(self, _IMP_misc.new_LogPairScore())

    def do_get_inputs(self, arg2, arg3):
        r"""do_get_inputs(LogPairScore self, Model arg2, IMP::ParticleIndexes const & arg3) -> IMP::ModelObjectsTemp"""
        return _IMP_misc.LogPairScore_do_get_inputs(self, arg2, arg3)

    def get_version_info(self):
        r"""get_version_info(LogPairScore self) -> VersionInfo"""
        return _IMP_misc.LogPairScore_get_version_info(self)

    def get_particle_pairs(self):
        r"""get_particle_pairs(LogPairScore self) -> IMP::ParticlePairsTemp"""
        return _IMP_misc.LogPairScore_get_particle_pairs(self)

    def clear(self):
        r"""clear(LogPairScore self)"""
        return _IMP_misc.LogPairScore_clear(self)

    def get_contains(self, pp):
        r"""get_contains(LogPairScore self, IMP::ParticlePair const & pp) -> bool"""
        return _IMP_misc.LogPairScore_get_contains(self, pp)

    def __str__(self):
        r"""__str__(LogPairScore self) -> std::string"""
        return _IMP_misc.LogPairScore___str__(self)

    def __repr__(self):
        r"""__repr__(LogPairScore self) -> std::string"""
        return _IMP_misc.LogPairScore___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_LogPairScore(o)


# Register LogPairScore in _IMP_misc:
_IMP_misc.LogPairScore_swigregister(LogPairScore)

class StateAdaptor(IMP.ScoreState):
    r"""Proxy of C++ IMP::misc::StateAdaptor class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, m, before, after):
        r"""__init__(StateAdaptor self, Model m, OptimizerState before, OptimizerState after) -> StateAdaptor"""
        _IMP_misc.StateAdaptor_swiginit(self, _IMP_misc.new_StateAdaptor(m, before, after))

    def do_before_evaluate(self):
        r"""do_before_evaluate(StateAdaptor self)"""
        return _IMP_misc.StateAdaptor_do_before_evaluate(self)

    def do_after_evaluate(self, da):
        r"""do_after_evaluate(StateAdaptor self, DerivativeAccumulator da)"""
        return _IMP_misc.StateAdaptor_do_after_evaluate(self, da)

    def do_get_inputs(self):
        r"""do_get_inputs(StateAdaptor self) -> IMP::ModelObjectsTemp"""
        return _IMP_misc.StateAdaptor_do_get_inputs(self)

    def do_get_outputs(self):
        r"""do_get_outputs(StateAdaptor self) -> IMP::ModelObjectsTemp"""
        return _IMP_misc.StateAdaptor_do_get_outputs(self)

    def get_version_info(self):
        r"""get_version_info(StateAdaptor self) -> VersionInfo"""
        return _IMP_misc.StateAdaptor_get_version_info(self)

    def __str__(self):
        r"""__str__(StateAdaptor self) -> std::string"""
        return _IMP_misc.StateAdaptor___str__(self)

    def __repr__(self):
        r"""__repr__(StateAdaptor self) -> std::string"""
        return _IMP_misc.StateAdaptor___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_StateAdaptor(o)


# Register StateAdaptor in _IMP_misc:
_IMP_misc.StateAdaptor_swigregister(StateAdaptor)

class CommonEndpointPairFilter(IMP.PairPredicate):
    r"""Proxy of C++ IMP::misc::CommonEndpointPairFilter class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self):
        r"""__init__(CommonEndpointPairFilter self) -> CommonEndpointPairFilter"""
        _IMP_misc.CommonEndpointPairFilter_swiginit(self, _IMP_misc.new_CommonEndpointPairFilter())

    def do_get_inputs(self, m, pis):
        r"""do_get_inputs(CommonEndpointPairFilter self, Model m, IMP::ParticleIndexes const & pis) -> IMP::ModelObjectsTemp"""
        return _IMP_misc.CommonEndpointPairFilter_do_get_inputs(self, m, pis)

    def get_value(self, *args):
        r"""
        get_value(CommonEndpointPairFilter self, IMP::ParticlePair const & a) -> int
        get_value(CommonEndpointPairFilter self, IMP::ParticlePairsTemp const & o) -> IMP::Ints
        """
        return _IMP_misc.CommonEndpointPairFilter_get_value(self, *args)

    def get_value_index(self, *args):
        r"""
        get_value_index(CommonEndpointPairFilter self, Model m, IMP::ParticleIndexPair const & p) -> int
        get_value_index(CommonEndpointPairFilter self, Model m, IMP::ParticleIndexPairs const & o) -> IMP::Ints
        """
        return _IMP_misc.CommonEndpointPairFilter_get_value_index(self, *args)

    def get_version_info(self):
        r"""get_version_info(CommonEndpointPairFilter self) -> VersionInfo"""
        return _IMP_misc.CommonEndpointPairFilter_get_version_info(self)

    def __str__(self):
        r"""__str__(CommonEndpointPairFilter self) -> std::string"""
        return _IMP_misc.CommonEndpointPairFilter___str__(self)

    def __repr__(self):
        r"""__repr__(CommonEndpointPairFilter self) -> std::string"""
        return _IMP_misc.CommonEndpointPairFilter___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_CommonEndpointPairFilter(o)


# Register CommonEndpointPairFilter in _IMP_misc:
_IMP_misc.CommonEndpointPairFilter_swigregister(CommonEndpointPairFilter)

class CustomXYZR(IMP.core.XYZ):
    r"""Proxy of C++ IMP::misc::CustomXYZR class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def get_decorator_traits(self):
        r"""get_decorator_traits(CustomXYZR self) -> FloatKey"""
        return _IMP_misc.CustomXYZR_get_decorator_traits(self)

    @staticmethod
    def get_default_decorator_traits():
        r"""get_default_decorator_traits() -> FloatKey"""
        return _IMP_misc.CustomXYZR_get_default_decorator_traits()

    def __init__(self, *args):
        r"""
        __init__(CustomXYZR self) -> CustomXYZR
        __init__(CustomXYZR self, Model m, ParticleIndex id, FloatKey tr=IMP::core::XYZR::get_radius_key()) -> CustomXYZR
        __init__(CustomXYZR self, _ParticleAdaptor d, FloatKey tr=IMP::core::XYZR::get_radius_key()) -> CustomXYZR
        """
        _IMP_misc.CustomXYZR_swiginit(self, _IMP_misc.new_CustomXYZR(*args))

    def show(self, *args):
        r"""show(CustomXYZR self, _ostream out=std::cout)"""
        return _IMP_misc.CustomXYZR_show(self, *args)

    @staticmethod
    def setup_particle(*args):
        r"""
        setup_particle(Model m, ParticleIndex pi, FloatKey tr=IMP::misc::CustomXYZR::get_default_decorator_traits()) -> CustomXYZR
        setup_particle(_ParticleAdaptor d, FloatKey tr=IMP::misc::CustomXYZR::get_default_decorator_traits()) -> CustomXYZR
        setup_particle(Model m, ParticleIndex pi, double radius, FloatKey tr=IMP::misc::CustomXYZR::get_default_decorator_traits()) -> CustomXYZR
        setup_particle(_ParticleAdaptor d, double radius, FloatKey tr=IMP::misc::CustomXYZR::get_default_decorator_traits()) -> CustomXYZR
        setup_particle(Model m, ParticleIndex pi, Sphere3D ball, FloatKey tr=IMP::misc::CustomXYZR::get_default_decorator_traits()) -> CustomXYZR
        setup_particle(_ParticleAdaptor d, Sphere3D ball, FloatKey tr=IMP::misc::CustomXYZR::get_default_decorator_traits()) -> CustomXYZR
        """
        return _IMP_misc.CustomXYZR_setup_particle(*args)

    @staticmethod
    def get_is_setup(*args):
        r"""
        get_is_setup(_ParticleAdaptor p, FloatKey tr=IMP::core::XYZR::get_radius_key()) -> bool
        get_is_setup(Model m, ParticleIndex pi, FloatKey radius_key=get_default_radius_key()) -> bool
        """
        return _IMP_misc.CustomXYZR_get_is_setup(*args)

    def get_radius(self):
        r"""get_radius(CustomXYZR self) -> IMP::Float"""
        return _IMP_misc.CustomXYZR_get_radius(self)

    def set_radius(self, t):
        r"""set_radius(CustomXYZR self, IMP::Float t)"""
        return _IMP_misc.CustomXYZR_set_radius(self, t)

    def get_sphere(self):
        r"""get_sphere(CustomXYZR self) -> Sphere3D"""
        return _IMP_misc.CustomXYZR_get_sphere(self)

    def set_sphere(self, s):
        r"""set_sphere(CustomXYZR self, Sphere3D s)"""
        return _IMP_misc.CustomXYZR_set_sphere(self, s)

    @staticmethod
    def get_default_radius_key():
        r"""get_default_radius_key() -> FloatKey"""
        return _IMP_misc.CustomXYZR_get_default_radius_key()

    def add_to_radius_derivative(self, v, d):
        r"""add_to_radius_derivative(CustomXYZR self, double v, DerivativeAccumulator d)"""
        return _IMP_misc.CustomXYZR_add_to_radius_derivative(self, v, d)

    def get_traits(self):
        r"""get_traits(CustomXYZR self) -> FloatKey"""
        return _IMP_misc.CustomXYZR_get_traits(self)

    def add_attribute(self, *args):
        r"""
        add_attribute(CustomXYZR self, FloatKey k, IMP::Float v, bool opt)
        add_attribute(CustomXYZR self, FloatKey a0, IMP::Float a1)
        add_attribute(CustomXYZR self, IntKey a0, IMP::Int a1)
        add_attribute(CustomXYZR self, FloatsKey a0, IMP::Floats a1)
        add_attribute(CustomXYZR self, IntsKey a0, IMP::Ints a1)
        add_attribute(CustomXYZR self, StringKey a0, IMP::String a1)
        add_attribute(CustomXYZR self, ParticleIndexKey a0, Particle a1)
        add_attribute(CustomXYZR self, ObjectKey a0, Object a1)
        """
        return _IMP_misc.CustomXYZR_add_attribute(self, *args)

    def get_value(self, *args):
        r"""
        get_value(CustomXYZR self, FloatKey a0) -> IMP::Float
        get_value(CustomXYZR self, IntKey a0) -> IMP::Int
        get_value(CustomXYZR self, FloatsKey a0) -> IMP::Floats
        get_value(CustomXYZR self, IntsKey a0) -> IMP::Ints
        get_value(CustomXYZR self, StringKey a0) -> IMP::String
        get_value(CustomXYZR self, ParticleIndexKey a0) -> Particle
        get_value(CustomXYZR self, ObjectKey a0) -> Object
        """
        return _IMP_misc.CustomXYZR_get_value(self, *args)

    def set_value(self, *args):
        r"""
        set_value(CustomXYZR self, FloatKey a0, IMP::Float a1)
        set_value(CustomXYZR self, IntKey a0, IMP::Int a1)
        set_value(CustomXYZR self, FloatsKey a0, IMP::Floats a1)
        set_value(CustomXYZR self, IntsKey a0, IMP::Ints a1)
        set_value(CustomXYZR self, StringKey a0, IMP::String a1)
        set_value(CustomXYZR self, ParticleIndexKey a0, Particle a1)
        set_value(CustomXYZR self, ObjectKey a0, Object a1)
        """
        return _IMP_misc.CustomXYZR_set_value(self, *args)

    def remove_attribute(self, *args):
        r"""
        remove_attribute(CustomXYZR self, FloatKey a0)
        remove_attribute(CustomXYZR self, IntKey a0)
        remove_attribute(CustomXYZR self, FloatsKey a0)
        remove_attribute(CustomXYZR self, IntsKey a0)
        remove_attribute(CustomXYZR self, StringKey a0)
        remove_attribute(CustomXYZR self, ParticleIndexKey a0)
        remove_attribute(CustomXYZR self, ObjectKey a0)
        """
        return _IMP_misc.CustomXYZR_remove_attribute(self, *args)

    def has_attribute(self, *args):
        r"""
        has_attribute(CustomXYZR self, FloatKey a0) -> bool
        has_attribute(CustomXYZR self, IntKey a0) -> bool
        has_attribute(CustomXYZR self, FloatsKey a0) -> bool
        has_attribute(CustomXYZR self, IntsKey a0) -> bool
        has_attribute(CustomXYZR self, StringKey a0) -> bool
        has_attribute(CustomXYZR self, ParticleIndexKey a0) -> bool
        has_attribute(CustomXYZR self, ObjectKey a0) -> bool
        """
        return _IMP_misc.CustomXYZR_has_attribute(self, *args)

    def get_derivative(self, a0):
        r"""get_derivative(CustomXYZR self, FloatKey a0) -> double"""
        return _IMP_misc.CustomXYZR_get_derivative(self, a0)

    def get_name(self):
        r"""get_name(CustomXYZR self) -> std::string"""
        return _IMP_misc.CustomXYZR_get_name(self)

    def clear_caches(self):
        r"""clear_caches(CustomXYZR self)"""
        return _IMP_misc.CustomXYZR_clear_caches(self)

    def set_name(self, a0):
        r"""set_name(CustomXYZR self, std::string a0)"""
        return _IMP_misc.CustomXYZR_set_name(self, a0)

    def set_check_level(self, a0):
        r"""set_check_level(CustomXYZR self, IMP::CheckLevel a0)"""
        return _IMP_misc.CustomXYZR_set_check_level(self, a0)

    def add_to_derivative(self, a0, a1, a2):
        r"""add_to_derivative(CustomXYZR self, FloatKey a0, double a1, DerivativeAccumulator a2)"""
        return _IMP_misc.CustomXYZR_add_to_derivative(self, a0, a1, a2)

    def set_is_optimized(self, a0, a1):
        r"""set_is_optimized(CustomXYZR self, FloatKey a0, bool a1)"""
        return _IMP_misc.CustomXYZR_set_is_optimized(self, a0, a1)

    def get_is_optimized(self, a0):
        r"""get_is_optimized(CustomXYZR self, FloatKey a0) -> bool"""
        return _IMP_misc.CustomXYZR_get_is_optimized(self, a0)

    def get_check_level(self):
        r"""get_check_level(CustomXYZR self) -> IMP::CheckLevel"""
        return _IMP_misc.CustomXYZR_get_check_level(self)

    def __eq__(self, *args):
        r"""
        __eq__(CustomXYZR self, CustomXYZR o) -> bool
        __eq__(CustomXYZR self, Particle d) -> bool
        """
        return _IMP_misc.CustomXYZR___eq__(self, *args)

    def __ne__(self, *args):
        r"""
        __ne__(CustomXYZR self, CustomXYZR o) -> bool
        __ne__(CustomXYZR self, Particle d) -> bool
        """
        return _IMP_misc.CustomXYZR___ne__(self, *args)

    def __le__(self, *args):
        r"""
        __le__(CustomXYZR self, CustomXYZR o) -> bool
        __le__(CustomXYZR self, Particle d) -> bool
        """
        return _IMP_misc.CustomXYZR___le__(self, *args)

    def __lt__(self, *args):
        r"""
        __lt__(CustomXYZR self, CustomXYZR o) -> bool
        __lt__(CustomXYZR self, Particle d) -> bool
        """
        return _IMP_misc.CustomXYZR___lt__(self, *args)

    def __ge__(self, *args):
        r"""
        __ge__(CustomXYZR self, CustomXYZR o) -> bool
        __ge__(CustomXYZR self, Particle d) -> bool
        """
        return _IMP_misc.CustomXYZR___ge__(self, *args)

    def __gt__(self, *args):
        r"""
        __gt__(CustomXYZR self, CustomXYZR o) -> bool
        __gt__(CustomXYZR self, Particle d) -> bool
        """
        return _IMP_misc.CustomXYZR___gt__(self, *args)

    def __hash__(self):
        r"""__hash__(CustomXYZR self) -> std::size_t"""
        return _IMP_misc.CustomXYZR___hash__(self)

    def __str__(self):
        r"""__str__(CustomXYZR self) -> std::string"""
        return _IMP_misc.CustomXYZR___str__(self)

    def __repr__(self):
        r"""__repr__(CustomXYZR self) -> std::string"""
        return _IMP_misc.CustomXYZR___repr__(self)
    __swig_destroy__ = _IMP_misc.delete_CustomXYZR

# Register CustomXYZR in _IMP_misc:
_IMP_misc.CustomXYZR_swigregister(CustomXYZR)

def CustomXYZR_get_default_decorator_traits():
    r"""CustomXYZR_get_default_decorator_traits() -> FloatKey"""
    return _IMP_misc.CustomXYZR_get_default_decorator_traits()

def CustomXYZR_setup_particle(*args):
    r"""
    CustomXYZR_setup_particle(Model m, ParticleIndex pi, FloatKey tr=IMP::misc::CustomXYZR::get_default_decorator_traits()) -> CustomXYZR
    CustomXYZR_setup_particle(_ParticleAdaptor d, FloatKey tr=IMP::misc::CustomXYZR::get_default_decorator_traits()) -> CustomXYZR
    CustomXYZR_setup_particle(Model m, ParticleIndex pi, double radius, FloatKey tr=IMP::misc::CustomXYZR::get_default_decorator_traits()) -> CustomXYZR
    CustomXYZR_setup_particle(_ParticleAdaptor d, double radius, FloatKey tr=IMP::misc::CustomXYZR::get_default_decorator_traits()) -> CustomXYZR
    CustomXYZR_setup_particle(Model m, ParticleIndex pi, Sphere3D ball, FloatKey tr=IMP::misc::CustomXYZR::get_default_decorator_traits()) -> CustomXYZR
    CustomXYZR_setup_particle(_ParticleAdaptor d, Sphere3D ball, FloatKey tr=IMP::misc::CustomXYZR::get_default_decorator_traits()) -> CustomXYZR
    """
    return _IMP_misc.CustomXYZR_setup_particle(*args)

def CustomXYZR_get_is_setup(*args):
    r"""
    CustomXYZR_get_is_setup(_ParticleAdaptor p, FloatKey tr=IMP::core::XYZR::get_radius_key()) -> bool
    CustomXYZR_get_is_setup(Model m, ParticleIndex pi, FloatKey radius_key=get_default_radius_key()) -> bool
    """
    return _IMP_misc.CustomXYZR_get_is_setup(*args)

def CustomXYZR_get_default_radius_key():
    r"""CustomXYZR_get_default_radius_key() -> FloatKey"""
    return _IMP_misc.CustomXYZR_get_default_radius_key()


def __lshift__(out, n):
    r"""__lshift__(_ostream out, CustomXYZR n) -> _ostream"""
    return _IMP_misc.__lshift__(out, n)

def get_distance(a, b):
    r"""get_distance(CustomXYZR a, CustomXYZR b) -> double"""
    return _IMP_misc.get_distance(a, b)

def get_sphere_d_geometry(d):
    r"""get_sphere_d_geometry(CustomXYZR d) -> Sphere3D"""
    return _IMP_misc.get_sphere_d_geometry(d)

def set_sphere_d_geometry(d, v):
    r"""set_sphere_d_geometry(CustomXYZR d, Sphere3D v)"""
    return _IMP_misc.set_sphere_d_geometry(d, v)
class SoftCylinderPairScore(IMP.PairScore):
    r"""Proxy of C++ IMP::misc::SoftCylinderPairScore class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, k):
        r"""__init__(SoftCylinderPairScore self, double k) -> SoftCylinderPairScore"""
        _IMP_misc.SoftCylinderPairScore_swiginit(self, _IMP_misc.new_SoftCylinderPairScore(k))

    def do_get_inputs(self, m, pis):
        r"""do_get_inputs(SoftCylinderPairScore self, Model m, IMP::ParticleIndexes const & pis) -> IMP::ModelObjectsTemp"""
        return _IMP_misc.SoftCylinderPairScore_do_get_inputs(self, m, pis)

    def get_version_info(self):
        r"""get_version_info(SoftCylinderPairScore self) -> VersionInfo"""
        return _IMP_misc.SoftCylinderPairScore_get_version_info(self)

    def __str__(self):
        r"""__str__(SoftCylinderPairScore self) -> std::string"""
        return _IMP_misc.SoftCylinderPairScore___str__(self)

    def __repr__(self):
        r"""__repr__(SoftCylinderPairScore self) -> std::string"""
        return _IMP_misc.SoftCylinderPairScore___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_SoftCylinderPairScore(o)


# Register SoftCylinderPairScore in _IMP_misc:
_IMP_misc.SoftCylinderPairScore_swigregister(SoftCylinderPairScore)

class DecayPairContainerOptimizerState(IMP.OptimizerState):
    r"""Proxy of C++ IMP::misc::DecayPairContainerOptimizerState class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args):
        r"""__init__(DecayPairContainerOptimizerState self, Model m, PairPredicate pred, IMP::ParticleIndexPairs const & initial_list, std::string name="DecayPairContainerOptimizerState%1%") -> DecayPairContainerOptimizerState"""
        _IMP_misc.DecayPairContainerOptimizerState_swiginit(self, _IMP_misc.new_DecayPairContainerOptimizerState(*args))

    def get_output_container(self):
        r"""get_output_container(DecayPairContainerOptimizerState self) -> PairContainer"""
        return _IMP_misc.DecayPairContainerOptimizerState_get_output_container(self)

    def do_update(self, call_num):
        r"""do_update(DecayPairContainerOptimizerState self, unsigned int call_num)"""
        return _IMP_misc.DecayPairContainerOptimizerState_do_update(self, call_num)

    def get_version_info(self):
        r"""get_version_info(DecayPairContainerOptimizerState self) -> VersionInfo"""
        return _IMP_misc.DecayPairContainerOptimizerState_get_version_info(self)

    def __str__(self):
        r"""__str__(DecayPairContainerOptimizerState self) -> std::string"""
        return _IMP_misc.DecayPairContainerOptimizerState___str__(self)

    def __repr__(self):
        r"""__repr__(DecayPairContainerOptimizerState self) -> std::string"""
        return _IMP_misc.DecayPairContainerOptimizerState___repr__(self)

    @staticmethod
    def get_from(o):
       return _object_cast_to_DecayPairContainerOptimizerState(o)


# Register DecayPairContainerOptimizerState in _IMP_misc:
_IMP_misc.DecayPairContainerOptimizerState_swigregister(DecayPairContainerOptimizerState)


def get_module_version():
    r"""get_module_version() -> std::string const"""
    return _IMP_misc.get_module_version()

def get_example_path(fname):
    r"""get_example_path(std::string fname) -> std::string"""
    return _IMP_misc.get_example_path(fname)

def get_data_path(fname):
    r"""get_data_path(std::string fname) -> std::string"""
    return _IMP_misc.get_data_path(fname)

from . import _version_check
_version_check.check_version(get_module_version())
__version__ = get_module_version()






import sys
import os


def is_disabled(fname):
    with open(fname) as fh:
        for line in fh:
            if line.startswith('ok=True'):
                return False
    return True


def get_disabled_modules():
    subdir = 'build_info'
    for f in os.listdir(subdir):
        if f.startswith('IMP.'):
            if is_disabled(os.path.join(subdir, f)):
                yield f[4:]


expected_disabled = frozenset(sys.argv[1].split(':'))

disabled = frozenset(get_disabled_modules())

if disabled != expected_disabled:
    raise ValueError("Expecting %s to be disabled, found %s"
                     % (sorted(expected_disabled), sorted(disabled)))



#!/usr/bin/env python

from __future__ import print_function
import IMP.multifit
from IMP import ArgumentParser

__doc__ = "Add RMSD to reference to each fitting file."

def parse_args():
    desc =  """
Given a set of local fits (e.g. generated by fit_fft), the RMSD between each
subunit and a reference orientation is calculated and added to each fitting
file, in the final "RMSD to reference" column. (The original fitting file is
not modified; a new fitting file is created with a '.RMSD' extension.)

Note that the assembly input file must contain a reference PDB filename for
each subunit (in the rightmost column).
"""
    p = ArgumentParser(description=desc)
    p.add_argument("-d", action="store_true", dest="use_dock",
                   help="if set the docking transformation is used (and not "
                        "the fitting transformation)")
    p.add_argument("assembly_file", help="assembly file name")
    p.add_argument("proteomics_file", help="proteomics file name")
    p.add_argument("mapping_file", help="mapping file name")
    p.add_argument("param_file", help="parameter file name")
    return p.parse_args()


def run(asmb_fn, proteomics_fn, mapping_fn, params_fn, dock_trans):
    asmb = IMP.multifit.read_settings(asmb_fn)

    prot_data = IMP.multifit.read_proteomics_data(proteomics_fn)
    print("=========3")
    mapping_data = IMP.multifit.read_protein_anchors_mapping(
        prot_data, mapping_fn)
    alignment_params = IMP.multifit.AlignmentParams(params_fn)
    align = IMP.multifit.ProteomicsEMAlignmentAtomic(
        mapping_data, asmb, alignment_params)
    ensmb = IMP.multifit.Ensemble(asmb, mapping_data)
    print("=========6")
    # load all proteomics restraints
    mdl = align.get_model()
    mhs = align.get_molecules()
    gs = []
    for i, mh in enumerate(mhs):
        fits_fn = asmb.get_component_header(i).get_transformations_fn()
        fits = IMP.multifit.read_fitting_solutions(fits_fn)
        print("calculating rmsd for", len(fits), "fits of protein", mh.get_name())
        xyz = IMP.core.XYZs(IMP.core.get_leaves(mh))
        mh_ref = IMP.atom.read_pdb(
            asmb.get_component_header(i).get_reference_fn(),
            mdl)
        rb = IMP.atom.create_rigid_body(mh_ref)
        xyz_ref = IMP.core.XYZs(IMP.core.get_leaves(mh_ref))
        new_fits = []
        for i, rec in enumerate(fits):
            fit_t = rec.get_fit_transformation()
            if dock_trans:
                fit_t = rec.get_dock_transformation()
            IMP.core.transform(rb, fit_t)
            rmsd = IMP.atom.get_rmsd(xyz, xyz_ref)
            rec.set_rmsd_to_reference(rmsd)
            new_fits.append(rec)
            IMP.core.transform(rb, fit_t.get_inverse())
        IMP.multifit.write_fitting_solutions(fits_fn + ".RMSD", new_fits)


def main():
    args = parse_args()
    run(args.assembly_file, args.proteomics_file, args.mapping_file,
        args.param_file, args.use_dock)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

"""
Set up a module by
- checking that all required modules and dependencies are found
- creating the config header and .cpp and version check .py
- linking .py files from the bin and benchmarks directories into the build dir
- Linking Python apps into the build dir.

If the module cannot be configured, the script exits with an error.
"""

import sys
from optparse import OptionParser
import os.path
import tools
import glob
import pickle
import re

TOPDIR = os.path.abspath(os.path.dirname(__file__))

parser = OptionParser()
parser.add_option("--build_dir", help="IMP build directory", default=None)
parser.add_option("-D", "--defines", dest="defines", default="",
                  help="Colon separated list of defines.")
parser.add_option("-n", "--name",
                  dest="name", help="The name of the module.")
parser.add_option("-s", "--source",
                  dest="source", help="The root for IMP source.")
parser.add_option("-d", "--datapath",
                  dest="datapath", default="", help="An extra IMP datapath.")


def add_list_to_defines(cppdefines, data, sym, val, names):
    for n in sorted(names):
        nn = n.replace(".", "_").replace("-", "_").upper()
        cppdefines.append("#define IMP_%s_%s_%s"
                          % (data["name"].upper(), sym, nn))
        cppdefines.append("#define IMP_%s_HAS_%s %d"
                          % (data["name"].upper(), nn, val))


def make_header(options, module):
    if module.python_only:
        return
    if module.name == 'kernel':
        dir = os.path.join("include", "IMP")
    else:
        dir = os.path.join("include", "IMP", module.name)
    file = os.path.join(dir, "%s_config.h" % module.name)
    header_template = tools.CPPFileGenerator(
        os.path.join(TOPDIR, "config_templates", "header.h"))
    try:
        os.makedirs(dir)
    except OSError:
        # exists
        pass

    data = {}
    data["name"] = module.name
    if module.name == 'kernel':
        data["namespace"] = "IMP"
        data["begin_ns"] = "namespace IMP{"
        data["end_ns"] = "}"
        data["filename"] = "IMP/%s_config.h" % module.name
    else:
        data["namespace"] = "IMP::%s" % module.name
        data["begin_ns"] = "namespace IMP{ namespace %s {" % module.name
        data["end_ns"] = "} }"
        data["filename"] = "IMP/%s/%s_config.h" % (module.name, module.name)
    data["cppprefix"] = "IMP%s" % module.name.upper().replace("_", "")
    if data["name"] != "kernel":
        data["showable"] = """#if !defined(IMP_DOXYGEN) && !defined(SWIG)

#include <IMP/Showable.h>
#include <IMP/hash.h>

%(begin_ns)s
using ::IMP::Showable;
using ::IMP::operator<<;
using ::IMP::hash_value;
%(end_ns)s // namespace
%(begin_ns)s namespace internal {
using ::IMP::Showable;
using ::IMP::operator<<;
using ::IMP::hash_value;
} %(end_ns)s // namespace

#endif // !defined(SWIG) && !defined(IMP_DOXYGEN)
""" % data
    else:
        data["showable"] = ""
    cppdefines = []
    if options.defines != "":
        for define in tools.split(options.defines):
            parts = define.split("=")
            if len(parts) == 2:
                cppdefines.append("#define %s %s" % (parts[0], parts[1]))
            else:
                cppdefines.append("#define %s" % parts[0])

    cf = module.configured

    optional_modules = [x for x in cf.modules
                        if x not in module.required_modules]
    optional_dependencies = [x for x in cf.dependencies
                             if x not in module.required_dependencies]
    add_list_to_defines(cppdefines, data, "USE", 1,
                        ["imp_" + x.name for x in optional_modules])
    add_list_to_defines(cppdefines, data, "NO", 0,
                        ["imp_" + x.name for x in cf.unfound_modules])
    add_list_to_defines(cppdefines, data, "USE", 1, optional_dependencies)
    add_list_to_defines(cppdefines, data, "NO", 0, cf.unfound_dependencies)
    data["cppdefines"] = "\n".join(cppdefines)
    header_template.write(file, data)


class ModuleDoxFileGenerator(tools.FileGenerator):
    def __init__(self, template_file, module, modules, finder):
        self.module = module
        self.modules = modules
        self.finder = finder
        tools.FileGenerator.__init__(self, template_file, '#')

    def get_output_file_contents(self, options):
        template = self.template
        module = self.module
        modules = self.modules
        template = template.replace("@IMP_SOURCE_PATH@", options.source)
        template = template.replace("@VERSION@", "NONE")
        template = template.replace("@NAME@", module.name)
        template = template.replace("@PROJECT_BRIEF@",
                                    '"The Integrative Modeling Platform"')
        template = template.replace("@RECURSIVE@", "YES")
        template = template.replace("@EXCLUDE_PATTERNS@", "*/tutorial/*")
        template = template.replace("@IS_HTML@", "NO")
        template = template.replace("@IS_XML@", "YES")
        template = template.replace("@PROJECT_NAME@", "IMP." + module.name)
        template = template.replace("@HTML_OUTPUT@", "../../doc/html/"
                                    + module.name)
        template = template.replace("@XML_OUTPUT@", "xml")
        template = template.replace("@TREEVIEW@", "NO")
        template = template.replace("@GENERATE_TAGFILE@", "tags")
        template = template.replace(
            "@LAYOUT_FILE@",
            "%s/doc/doxygen/module_layout.xml" % options.source)
        template = template.replace("@MAINPAGE@", "README.md")
        template = template.replace("@INCLUDE_PATH@", "include")
        template = template.replace("@FILE_PATTERNS@",
                                    "*.cpp *.h *.py *.md *.dox")
        template = template.replace("@WARNINGS@", "warnings.txt")
        # include lib and doxygen in input
        inputs = []
        if module.name == "kernel":
            inputs.append("lib/IMP/")
            inputs.append("include/IMP/")
            exclude = ["include/IMP/%s include/IMP/%s.h lib/IMP/%s"
                       % (m, m, m) for m in sorted(self.finder.keys())]
            exclude.append("include/IMP/base include/IMP/base.h lib/IMP/base")
            template = template.replace(
                "@EXCLUDE@", " \\\n                         ".join(exclude))
        else:
            template = template.replace("@EXCLUDE@", "")
            inputs.append("include/IMP/" + module.name)
            inputs.append("lib/IMP/" + module.name)
        inputs.append("examples/" + module.name)
        # suppress a warning since git removes empty dirs and doxygen
        # gets confused if the input path doesn't exist
        docpath = os.path.join(module.path, "doc")
        if os.path.exists(docpath):
            inputs.append(docpath)
        # overview for module
        inputs.append("../generated/IMP_%s.dox" % module.name)
        template = template.replace(
            "@INPUT_PATH@", " \\\n                         ".join(inputs))
        tags = [os.path.join(options.source, 'doc', 'doxygen',
                             'dummy_module_tags.xml')]
        for m in modules:
            tags.append(os.path.join("../", m.name, "tags")
                        + "=" + "../" + m.name)
        template = template.replace(
            "@TAGS@", " \\\n                         ".join(tags))
        if module.name == "example":
            template = template.replace(
                "@EXAMPLE_PATH@",
                "examples/example %s/modules/example" % options.source)
        else:
            template = template.replace("@EXAMPLE_PATH@",
                                        "examples/" + module.name)
        return template


def make_doxygen(options, module, modules, finder):
    file = os.path.join("doxygen", module.name, "Doxyfile")
    g = ModuleDoxFileGenerator(
        os.path.join(TOPDIR, "doxygen_templates", "Doxyfile.in"),
        module, modules, finder)
    g.write(file, options)


def write_no_ok(module):
    new_order = [x for x in tools.get_sorted_order() if x != module]
    tools.set_sorted_order(new_order)
    tools.rewrite(os.path.join("build_info", "IMP." + module),
                  "ok=False\n", verbose=False)


def write_ok(module, modules, unfound_modules, dependencies,
             unfound_dependencies, swig_includes, swig_wrapper_includes,
             python_only):
    print("yes")
    config = ["ok=True"]
    modules = [x.name for x in modules]
    unfound_modules = [x.name for x in unfound_modules]
    vardict = locals()
    for varname in ("modules", "unfound_modules", "dependencies",
                    "unfound_dependencies", "swig_includes",
                    "swig_wrapper_includes"):
        var = vardict[varname]
        if len(var) > 0:
            config.append("%s = %s" % (varname, repr(":".join(var))))
    if python_only:
        config.append("python_only = True")
    tools.rewrite(os.path.join("build_info", "IMP." + module.name),
                  "\n".join(config))


def setup_module(module, finder):
    sys.stdout.write("Configuring module %s ..." % module.name)
    for d in module.required_dependencies:
        if not finder.get_dependency_info(d)["ok"]:
            print("Required dependency %s not found" % d)
            write_no_ok(module.name)
            return False, []
    dependencies = module.required_dependencies[:]
    unfound_dependencies = []
    for d in module.optional_dependencies:
        if finder.get_dependency_info(d)["ok"]:
            dependencies.append(d)
        else:
            unfound_dependencies.append(d)
    for d in module.required_modules:
        if not d.configured.ok:
            print("Required module IMP.%s not available" % d.name)
            write_no_ok(module.name)
            return False, []
    modules = module.required_modules[:]
    unfound_modules = []
    for d in module.optional_modules:
        if d.configured.ok:
            modules.append(d)
        else:
            unfound_modules.append(d)
    all_modules = finder.get_dependent_modules(modules)
    moddir = os.path.join('IMP',
                          '' if module.name == 'kernel' else module.name)
    swig_includes = \
        [os.path.split(x)[1]
         for x in tools.get_glob([os.path.join(module.path, "pyext",
                                               "include", "*.i")])] \
        + [os.path.join(moddir, os.path.split(x)[1])
           for x in tools.get_glob([os.path.join("include", moddir,
                                                 "*_macros.h")])]
    swig_wrapper_includes = [os.path.join(moddir, "internal",
                                          os.path.split(x)[1])
                             for x in tools.get_glob(
                                 [os.path.join(module.path,
                                               "include", "internal",
                                               "swig*.h")])]
    tools.mkdir(os.path.join("src", module.name))
    tools.mkdir(os.path.join("src", module.name + "_swig"))
    write_ok(module, all_modules, unfound_modules,
             finder.get_dependent_dependencies(all_modules, dependencies),
             unfound_dependencies, swig_includes, swig_wrapper_includes,
             module.python_only)
    return True, all_modules


def link_py_apps(module):
    path = os.path.join(module.path, "bin")
    tools.mkdir("bin", clean=False)
    bins = [b for b in glob.glob(os.path.join(path, '*'))
            if tools.filter_pyapps(b)]
    # rewrite Python shebang to use current version of Python (2 or 3)
    for source_bin in bins:
        contents = """#!%s
fname = '%s'
with open(fname) as fh:
    exec(compile(fh.read(), fname, 'exec'))
""" % (sys.executable, source_bin)
        dest_bin = os.path.join("bin", os.path.basename(source_bin))
        tools.rewrite(dest_bin, contents, verbose=False)
        os.chmod(dest_bin, 493)  # 493 = 0755, i.e. executable


def link_bin(options, module):
    path = os.path.join("module_bin", options.name)
    tools.mkdir(path, clean=False)
    for old in tools.get_glob([os.path.join(path, "*.py")]):
        os.unlink(old)
    tools.link_dir(os.path.join(module.path, "bin"),
                   path, clean=False, match=["*.py"])


def link_benchmark(options, module):
    path = os.path.join("benchmark", options.name)
    tools.mkdir(path, clean=False)
    for old in tools.get_glob([os.path.join(path, "*.py")]):
        os.unlink(old)
    tools.link_dir(os.path.join(module.path, "benchmark"),
                   path, clean=False, match=["*.py"])


def find_cmdline_links(mod, docdir, cmdline_tools):
    """Look for (sub)sections in the .dox or .md docs for each cmdline tool,
       and return a mapping from tool name to
       (doxygen link, brief desc, num)"""
    links = dict.fromkeys(cmdline_tools)
    num = 0
    todo = {}
    docre = re.compile(r'\\(subsection|section|page)\s+(\S+)\s+(\S+):\s*(.*)$')
    docre_sep = re.compile(r'\\(subsection|section|page)\s+(\S+)\s+(\S+)\s*$')
    mdre = re.compile(r'#*\s*(\S+):\s*([^#]+)#*\s*{#(\S+)}')
    mdre_sep = re.compile(r'#*\s*(\S+)\s*#*\s*{#(\S+)}')
    for g in [os.path.join(docdir, "README.md")] \
            + glob.glob(os.path.join(docdir, "doc", "*.dox")) \
            + glob.glob(os.path.join(docdir, "doc", "*.md")):
        for line in tools.open_utf8(g):
            if todo and len(line.rstrip('\r\n ')) > 0 \
               and line[0] not in " =-\\":
                (k, v) = todo.popitem()
                links[k] = (v, line.rstrip('\r\n '), num)
                num += 1
                todo = {}
            m = docre.search(line)
            if m and m.group(3) in links:
                links[m.group(3)] = (m.group(2), m.group(4), num)
                num += 1
            m = docre_sep.search(line)
            if m and m.group(3) in links:
                todo = {m.group(3): m.group(2)}
            m = mdre.search(line)
            if m and m.group(1) in links:
                links[m.group(1)] = (m.group(3), m.group(2), num)
                num += 1
            m = mdre_sep.search(line)
            if m and m.group(1) in links:
                todo = {m.group(1): m.group(2)}
    missing_links = [tool for tool, link in links.items() if link is None]
    if missing_links:
        print("""
Could not find section title for command line tool %s
in IMP.%s docs.

Each command line tool should have a section or page in the documentation
(in %s/README.md or
%s/doc/*.{dox,md})
that describes it. The section title should contain the tool's name and a
brief description (separated by a colon), followed by a unique doxygen ID.
Alternatively, the brief description can be given in the body immediately
following the title. For example, the tool do_foo.py could be documented with

\\section do_foo_bin do_foo.py: Do something with foo

or

\\section do_foo_bin do_foo.py
Do something with foo

in doxygen (\\subsection or \\page can also be used) or

doo_foo.py: Do something with foo {#do_foo_bin}
=================================

or

# doo_foo.py: Do something with foo {#do_foo_bin}

or

# doo_foo.py {#do_foo_bin}
Do something with foo

in Markdown.
""" % (", ".join(missing_links), mod, docdir, docdir))
        sys.exit(1)
    return links


def make_overview(module, cmdline_tools):
    cmdline_links = find_cmdline_links(module.name, module.path, cmdline_tools)
    pickle.dump(cmdline_links,
                open(os.path.join("build_info",
                                  "IMP_%s.pck" % module.name), 'wb'), -1)
    rmd = tools.open_utf8(os.path.join(module.path, "README.md"), "r").read()
    tools.rewrite(
        os.path.join("doxygen", "generated", "IMP_%s.dox" % module.name),
        """/** \\namespace %s
\\tableofcontents

%s
*/
""" % ('IMP' if module.name == 'kernel' else 'IMP::' + module.name, rmd))


def main():
    options, apps = parser.parse_args()
    with open("build_info/disabled", "r") as fh:
        disabled = tools.split(fh.read(), "\n")
    if options.name in disabled:
        print("%s is disabled" % options.name)
        write_no_ok(options.name)
        tools.rmdir(os.path.join("module_bin", options.name))
        tools.rmdir(os.path.join("benchmark", options.name))
        tools.rmdir(os.path.join("lib", "IMP", options.name))
        sys.exit(1)
    mf = tools.ModulesFinder(source_dir=options.source,
                             external_dir=options.build_dir,
                             module_name=options.name)
    module = mf[options.name]
    success, modules = setup_module(module, mf)
    if success:
        make_header(options, module)
        make_doxygen(options, module, modules, mf)
        make_overview(module, apps)
        link_bin(options, module)
        link_py_apps(module)
        link_benchmark(options, module)
        sys.exit(0)
    else:
        tools.rmdir(os.path.join("module_bin", options.name))
        tools.rmdir(os.path.join("benchmark", options.name))
        tools.rmdir(os.path.join("lib", "IMP", options.name))
        sys.exit(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

from __future__ import print_function
import IMP.multifit
from IMP import ArgumentParser

__doc__ = "Generate indexes of fitting solutions."

def parse_args():
    desc = """Generate indexes of fitting solutions."""
    p = ArgumentParser(description=desc)
    p.add_argument("assembly_name", help="name of the assembly")
    p.add_argument("assembly_file", help="assembly file name")
    p.add_argument("num_fits", type=int, help="number of fits")
    p.add_argument("indexes_file", help="indexes file name")
    return p.parse_args()


def run(assembly_name, asmb_fn, num_fits, mapping_fn=""):
    IMP.set_log_level(IMP.WARNING)
    asmb_input = IMP.multifit.read_settings(asmb_fn)
    asmb_input.set_was_used(True)
    ap_em_fn = asmb_input.get_assembly_header().get_coarse_over_sampled_ap_fn()
    if mapping_fn == "":
        mapping_fn = assembly_name + ".indexes.mapping.input"
    mapping_data = open(mapping_fn, "w")
    mapping_data.write("|anchors|" + ap_em_fn + "|\n")
    for i in range(asmb_input.get_number_of_component_headers()):
        name = asmb_input.get_component_header(i).get_name()
        trans_fn = asmb_input.get_component_header(i).get_transformations_fn()
        fits = IMP.multifit.read_fitting_solutions(trans_fn)
        print("number of fits for component", i, "is", len(fits), trans_fn)
        index_fn = assembly_name + "." + name + ".fit.indexes.txt"
        indexes = []
        for i in range(min(num_fits, len(fits))):
            indexes.append([i])
        IMP.multifit.write_paths(indexes, index_fn)
        mapping_data.write("|protein|" + name +
                           "|" + index_fn + "|\n")
    mapping_data.close()


def main():
    args = parse_args()
    run(args.assembly_name, args.assembly_file, args.num_fits,
        args.indexes_file)

if __name__ == "__main__":
    main()



## \example core/optimize_balls.py
# This example optimizes a set of a balls to form 100 chains packed into a
# box. It illustrates using Monte Carlo (incremental) and conjugate
# gradients in conjunction in a non-trivial optimization.

import IMP.core
import IMP.display
import IMP.container
import sys

IMP.setup_from_argv(sys.argv, "Optimize balls example")

bb = IMP.algebra.BoundingBox3D(IMP.algebra.Vector3D(0, 0, 0),
                               IMP.algebra.Vector3D(10, 10, 10))
if IMP.get_is_quick_test():
    ni = 2
    nj = 2
    np = 2
    radius = .45
    k = 100
    ncg = 10
    nmc = 1
    ninner = 1
    nouter = 1
else:
    ni = 10
    nj = 10
    np = 10
    radius = .45
    k = 100
    ncg = 1000
    nmc = ni * nj * np * 100
    ninner = 5
    nouter = 11

print(IMP.get_is_quick_test(), ni, nj, np, ninner, nouter)
# using a HarmonicDistancePairScore for fixed length links is more
# efficient than using a HarmonicSphereDistnacePairScore and works
# better with the optimizer
lps = IMP.core.HarmonicDistancePairScore(1.5 * radius, k)
sps = IMP.core.SoftSpherePairScore(k)

m = IMP.Model()
# IMP.set_log_level(IMP.SILENT)
aps = []
filters = []
movers = []
restraints = []
for i in range(0, ni):
    for j in range(0, nj):
        base = IMP.algebra.Vector3D(i, j, 0)
        chain = []
        for k in range(0, np):
            p = IMP.Particle(m)
            p.set_name("P" + str(i) + " " + str(j) + " " + str(k))
            s = IMP.algebra.Sphere3D(
                IMP.algebra.get_random_vector_in(bb), radius)
            d = IMP.core.XYZR.setup_particle(p, s)
            d.set_coordinates_are_optimized(True)
            movers.append(IMP.core.BallMover(m, p, radius * 2))
            movers[-1].set_was_used(True)
            IMP.display.Colored.setup_particle(
                p, IMP.display.get_display_color(i * nj + j))
            if k == 0:
                d.set_coordinates(base)
            else:
                d.set_coordinates_are_optimized(True)
            chain.append(p)
            aps.append(p)
        # set up a chain of bonds
        cpc = IMP.container.ExclusiveConsecutivePairContainer(m, chain)
        r = IMP.container.PairsRestraint(lps, cpc)
        restraints.append(r)

# don't apply excluded volume to consecutive particles
filters.append(IMP.container.ExclusiveConsecutivePairFilter())
ibss = IMP.core.BoundingBox3DSingletonScore(
    IMP.core.HarmonicUpperBound(0, k), bb)
bbr = IMP.container.SingletonsRestraint(ibss, aps)
restraints.append(bbr)

cg = IMP.core.ConjugateGradients(m)
mc = IMP.core.MonteCarlo(m)
mc.set_name("MC")
sm = IMP.core.SerialMover(movers)
mc.add_mover(sm)
# we are special casing the nbl term
isf = IMP.core.IncrementalScoringFunction(m, aps, restraints)
isf.set_name("I")
# use special incremental support for the non-bonded part
# apply the pair score sps to all touching ball pairs from the list of particles
# aps, using the filters to remove undersired pairs
# this is equivalent to the nbl construction above but optimized for
# incremental
isf.add_close_pair_score(sps, 0, aps, filters)

# create a scoring function for conjugate gradients that includes the
# ExcludedVolumeRestraint
nbl = IMP.core.ExcludedVolumeRestraint(aps, k, 1)
nbl.set_pair_filters(filters)
sf = IMP.core.RestraintsScoringFunction(restraints + [nbl], "RSF")

if True:
    mc.set_incremental_scoring_function(isf)
else:
    # we could, instead do non-incremental scoring
    mc.set_scoring_function(sf)

# first relax the bonds a bit
rs = []
for p in aps:
    rs.append(IMP.ScopedSetFloatAttribute(p, IMP.core.XYZR.get_radius_key(),
                                          0))
cg.set_scoring_function(sf)
cg.optimize(ncg)
for r in restraints:
    print(r.get_name(), r.evaluate(False))

# shrink each of the particles, relax the configuration, repeat
for i in range(1, nouter):
    rs = []
    factor = .1 * i
    for p in aps:
        rs.append(
            IMP.ScopedSetFloatAttribute(p, IMP.core.XYZR.get_radius_key(),
                                        IMP.core.XYZR(p).get_radius() * factor))
    # move each particle nmc times
    print(factor)
    for j in range(0, ninner):
        print("stage", j)
        mc.set_kt(100.0 / (3 * j + 1))
        print("mc", mc.optimize((j + 1) * nmc), cg.optimize(nmc))
    del rs
    for r in restraints:
        print(r.get_name(), r.evaluate(False))

w = IMP.display.PymolWriter("final.pym")
for p in aps:
    g = IMP.core.XYZRGeometry(p)
    w.add_geometry(g)
g = IMP.display.BoundingBoxGeometry(bb)
g.set_name("bb")
w.add_geometry(g)



from __future__ import print_function
import IMP
import IMP.em
import IMP.isd
import IMP.isd.gmm_tools
from IMP import ArgumentParser

import sys,os

def parse_args():
    desc = """
    Create a GMM from either density file (.mrc), a pdb file (.pdb)
    Will detect input format from extension.
    Outputs as text and optionally as a density map
    see help(-h)
"""
    p  = ArgumentParser(description=desc)

    p.add_argument("-t","--covar_type",dest="covar_type",default='full',
                      choices=['spherical', 'tied', 'diag', 'full'],
                      help="covariance type for the GMM")
    p.add_argument("-m","--out_map",dest="out_map",default='',
                      help="write out the gmm to an mrc file")
    p.add_argument("-a","--apix",dest="apix",default=1.0,type=float,
                      help="if you don't provide a map, set the voxel_size here (for sampling)")
    p.add_argument("-n","--num_samples",dest="num_samples",default=1000000,type=int,
                      help="num samples to draw from the density map")
    p.add_argument("-i","--num_iter",dest="num_iter",default=100,type=int,
                      help="num iterations of GMM")
    p.add_argument("-s","--threshold",dest="threshold",default=0.0,type=float,
                      help="threshold for the map before sampling")

    p.add_argument("-f","--force_radii",dest="force_radii",default=-1.0,
                      type=float,
                      help="force radii to be this value (spherical) -1 means deactivated ")
    p.add_argument("-w","--force_weight",dest="force_weight",default=-1.0,
                      type=float,
                      help="force weight to be this value (spherical) -1 means deactivated ")
    p.add_argument("-e", "--force_weight_frac", dest="force_weight_frac",
                   action="store_true", default=False,
                   help="force weight to be 1.0/(num centers). "
                        "Takes precedence over -w")
    p.add_argument("-d","--use_dirichlet",dest="use_dirichlet",default=False,
                      action="store_true",
                      help="use dirichlet process for fit")

    p.add_argument("-k","--multiply_by_mass",dest="multiply_by_mass",default=False,
                      action="store_true",
                      help="if set, will multiply all weights by the total mass of the particles (PDB ONLY)")
    p.add_argument("-x","--chain",dest="chain",default=None,
                      help="If you passed a PDB file, read this chain")

    p.add_argument("-z","--use_cpp",dest="use_cpp",default=False,
                      action="store_true",
                      help="EXPERIMENTAL. Uses the IMP GMM code. Requires isd_emxl")
    p.add_argument("data_file", help="data file name")
    p.add_argument("n_centers", type=int, help="number of centers")
    p.add_argument("out_file", help="output file name")
    return p.parse_args()

def run(args):
    data_fn = args.data_file
    ncenters = args.n_centers
    out_txt_fn = args.out_file
    mdl = IMP.Model()

    if not os.path.isfile(data_fn):
        raise Exception("The data file you entered: "+data_fn+" does not exist!")

    ### get points for fitting the GMM
    ext = data_fn.split('.')[-1]
    mass_multiplier = 1.0
    if ext=='pdb':
        mh = IMP.atom.read_pdb(data_fn,mdl,IMP.atom.NonWaterNonHydrogenPDBSelector())
        if args.chain:
            mps = IMP.atom.Selection(mh,chain=args.chain).get_selected_particles()
        else:
            mps = IMP.core.get_leaves(mh)

        if args.multiply_by_mass:
            mass_multiplier=sum(IMP.atom.Mass(p).get_mass() for p in mps)

        pts = [IMP.core.XYZ(p).get_coordinates() for p in mps]
        bbox = None
    elif ext=='mrc':
        dmap = IMP.em.read_map(data_fn,IMP.em.MRCReaderWriter())
        bbox = IMP.em.get_bounding_box(dmap)
        dmap.set_was_used(True)
        print('sampling points')
        pts = IMP.isd.sample_points_from_density(dmap,args.num_samples,args.threshold)
    else:
        raise ValueError("data_fn extension must be pdb or mrc")

    ### Do fitting to points
    if not args.use_cpp:
        density_ps = []
        print('fitting gmm')
        #IMP.isd_emxl.gmm_tools.draw_points(pts,'test_points.bild')

        if args.force_weight_frac:
            force_weight = 1.0/ncenters
        else:
            force_weight=args.force_weight
        if force_weight != -1:
            print('weight forced to',force_weight)
        if not args.use_dirichlet:
            gmm = IMP.isd.gmm_tools.fit_gmm_to_points(pts,ncenters,mdl,density_ps,
                                                      args.num_iter,args.covar_type,
                                                      force_radii=args.force_radii,
                                                      force_weight=args.force_weight,
                                                      mass_multiplier=mass_multiplier)
        else:
            gmm = IMP.isd.gmm_tools.fit_dirichlet_gmm_to_points(pts,ncenters,mdl,density_ps,
                                                                args.num_iter,args.covar_type,
                                                                mass_multiplier=mass_multiplier)

    else:
        try:
            import isd_emxl
        except ImportError:
            print("This option is experimental, only works if you have isd_emxl")
        gmm_threshold = 0.01
        density_ps = IMP.isd_emxl.fit_gaussians_to_density(mdl,dmap,args.num_samples,
                                                         ncenters,args.num_iter,
                                                         args.threshold,
                                                         gmm_threshold)

    ### Write to files
    comments = ['Created by create_gmm.py, IMP.isd version %s'
                % IMP.isd.get_module_version()]
    comments.append('data_fn: ' + IMP.get_relative_path(out_txt_fn, data_fn))
    comments.append('ncenters: %d' % ncenters)
    for key in ('covar_type', 'apix', 'num_samples', 'num_iter',
                'threshold', 'force_radii', 'force_weight',
                'force_weight_frac', 'use_dirichlet', 'multiply_by_mass',
                'chain'):
        comments.append('%s: %s' % (key, repr(getattr(args, key))))
    IMP.isd.gmm_tools.write_gmm_to_text(density_ps, out_txt_fn, comments)
    if args.out_map != '':
        IMP.isd.gmm_tools.write_gmm_to_map(density_ps, args.out_map,
                                           args.apix, bbox)

def main():
    args = parse_args()
    run(args)

if __name__=="__main__":
    main()



from __future__ import print_function
import glob
import os
import ast
import os.path
import shutil
import sys
import difflib
import subprocess
import signal

# cmake paths are always /-separated; on platforms where the path is not /
# (e.g. Windows) convert a path to or from a form cmake will like
if os.sep == '/':
    def to_cmake_path(p):
        return p

    def from_cmake_path(p):
        return p
else:
    def to_cmake_path(p):
        return p.replace(os.sep, '/')

    def from_cmake_path(p):
        return p.replace('/', os.sep)


def get_existing_links(path):
    """Get any symlinks in the given directory"""
    return [f for f in glob.glob(os.path.join(path, "*")) if os.path.islink(f)]


def mkdir(path, clean=True):
    if os.path.isdir(path):
        if clean:
            # remove any old links
            for f in glob.glob(os.path.join(path, "*")):
                if os.path.islink(f):
                    os.unlink(f)
            # Remove old lists of Python tests
            for f in glob.glob(os.path.join(path, "*.pytests")):
                os.unlink(f)
        else:
            # clean broken links
            for f in glob.glob(os.path.join(path, "*")):
                if os.path.islink(f) and not os.path.exists(os.readlink(f)):
                    os.unlink(f)
        return
    if os.path.isfile(path):
        os.unlink(path)
    os.makedirs(path)


def quote(st):
    out = st.replace(
        "\\f",
        ""). replace(
        "\\b",
        "").replace(
        "\\",
        "\\\\").replace(
        "\"",
        "\\\"")
    return out


def get_glob(patterns):
    ret = []
    for p in patterns:
        c = sorted(glob.glob(p))
        ret += c
    return ret


def rewrite(filename, contents, show_update=True, verbose=False):
    try:
        old = open(filename, "r").read()
        if old == contents:
            return
        elif verbose:
            print("    Different", filename)
            for ln in difflib.unified_diff(old.split("\n"),
                                           contents.split("\n")):
                stl = str(ln)
                if stl[0] in ('-', '+') and stl[1] not in ('-', '+'):
                    print("    " + stl)
    except Exception:
        pass
        # print "Missing", filename
    if show_update:
        print("Updating " + filename)
    dirpath = os.path.split(filename)[0]
    if dirpath != "":
        mkdir(dirpath, False)
    open(filename, "w").write(contents)


def rmdir(path):
    try:
        shutil.rmtree(path)
    except Exception:
        pass


def link(source, target, verbose=False):
    tpath = os.path.abspath(target)
    spath = os.path.abspath(source)
    # print tpath, spath
    if not os.path.exists(spath):
        if verbose:
            print("no source", spath)
        return
    if os.path.islink(tpath):
        if os.readlink(tpath) == spath:
            return
        else:
            os.unlink(tpath)
    elif os.path.isdir(tpath):
        shutil.rmtree(tpath)
    elif os.path.exists(tpath):
        os.unlink(tpath)
    if verbose:
        print("linking", spath, tpath)
    if hasattr(os, 'symlink'):
        os.symlink(spath, tpath)
    # Copy instead of link on platforms that don't support symlinks (Windows)
    elif os.path.isdir(spath):
        def excludes(src, names):
            return ['.svn', 'data']
        shutil.copytree(spath, tpath, ignore=excludes)
    else:
        shutil.copy(spath, tpath)


def link_dir(source_dir, target_dir, match=["*"], exclude=[],
             clean=True, verbose=False):
    if not isinstance(match, list):
        raise TypeError("match must be a list")
    exclude = exclude + ["SConscript", "CMakeLists.txt", ".svn"]
    # print "linking", source_dir, target_dir
    mkdir(target_dir, clean=False)
    if clean:
        existing_links = get_existing_links(target_dir)
    # Don't clean links here, as that forces any valid symlinks to be
    # recreated (potentially forcing a useless rebuild). We'll handle them
    # at the end of this function.
    files = []
    targets = {}
    for m in match:
        files.extend(get_glob([os.path.join(source_dir, m)]))
    for g in files:
        name = os.path.split(g)[1]
        if name not in exclude:
            target = os.path.join(target_dir, name)
            targets[target] = None
            link(g, target, verbose=verbose)
    if clean:
        # Remove any old links that are no longer valid
        for ln in existing_links:
            if ln not in targets:
                os.unlink(ln)


def get_modules(source):
    path = os.path.join(source, "modules", "*")
    globs = get_glob([path])
    mods = [(os.path.split(g)[1], g)
            for g in globs if os.path.split(g)[1] != "SConscript"
            and os.path.exists(os.path.join(g, "dependencies.py"))]
    return mods


# a version of split that doesn't return empty strings when there are no items


def split(string, sep=":"):
    return([x.replace("@", ":")
            for x in string.replace("\\:", "@").split(sep) if x != ""])


def get_project_info(path):
    cp = os.path.join(path, ".imp_info.py")
    if os.path.exists(cp):
        with open(cp) as fh:
            return ast.literal_eval(fh.read())
    elif path in ("", "/"):
        raise ValueError("no .imp_info.py found")
    else:
        return get_project_info(os.path.split(path)[0])


_subprocesses = []


def run_subprocess(command, **kwargs):
    global _subprocesses
    # if not kwargs.has_key("stdout"):
    #    kwargs["stdout"] = subprocess.PIPE
    # if not kwargs.has_key("stderr"):
    #    kwargs["stderr"] = subprocess.PIPE
    pro = subprocess.Popen(
        command, preexec_fn=os.setsid, stderr=subprocess.PIPE,
        stdout=subprocess.PIPE, universal_newlines=True, **kwargs)
    _subprocesses.append(pro)
    output, error = pro.communicate()
    ret = pro.returncode
    if ret != 0:
        sys.stderr.write(error + '\n')
        raise OSError("subprocess failed with return code %d: %s\n%s"
                      % (ret, " ".join(command), error))
    return output


def _sigHandler(signum, frame):
    print("starting handler")
    signal.signal(signal.SIGTERM, signal.SIG_DFL)
    global _subprocesses
    for p in _subprocesses:
        print("killing", p)
        try:
            os.kill(p.pid, signal.SIGTERM)
        except Exception:
            pass
    sys.exit(1)


signal.signal(signal.SIGTERM, _sigHandler)



## \example domino/custom_filter.py
# This example looks like the six particle optimization example except a
# filter is used instead of a restraint to remove the flip degree of
# freedom. The filter is written is Python, which makes for quick
# prototyping, but slower run times.

import IMP
import IMP.domino
import IMP.core
import IMP.container
import sys

IMP.setup_from_argv(sys.argv, "custom filter")

# set restraints


def create_scoring(m, ps):
    pairs = [[0, 1], [0, 2], [1, 3], [2, 3], [3, 4], [4, 5], [1, 5]]
    score = IMP.core.HarmonicDistancePairScore(1, 1)
    # the restraint will be broken apart during optimization
    pc = IMP.container.ListPairContainer(m,
                                         [(ps[p[0]], ps[p[1]]) for p in pairs],
                                         "Restrained pairs")
    pr = IMP.container.PairsRestraint(score, pc)
    pr.set_maximum_score(.01)
    d = IMP.core.DistanceToSingletonScore(IMP.core.HarmonicUpperBound(2, 1),
                                          IMP.algebra.Vector3D(2, 0, 0))
    return [pr]


def create_representation(m):
    ps = []
    for i in range(0, 6):
        p = m.add_particle("P%d" % i)
        IMP.core.XYZ.setup_particle(m, p, IMP.algebra.Vector3D(i, 0., 0.))
        ps.append(p)
    return ps


def create_discrete_states(m, ps):
    pst = IMP.domino.ParticleStatesTable()
    vs = [IMP.algebra.Vector3D(1, 0, 0),
          IMP.algebra.Vector3D(0, 1, 0),
          IMP.algebra.Vector3D(1, 1, 0),
          IMP.algebra.Vector3D(2, 1, 0),
          IMP.algebra.Vector3D(2, 0, 0)]
    vs = vs + [-v for v in vs]
    print(len(vs), "states for each particle")
    print(vs[1])
    states = IMP.domino.XYZStates(vs)
    # special case ps[0] to remove a sliding degree of freedom
    for p in ps[1:]:
        pst.set_particle_states(m.get_particle(p), states)
    return pst

# force particle p to be in state s


class MyFilterTable(IMP.domino.SubsetFilterTable):

    class MyFilter(IMP.domino.SubsetFilter):

        def __init__(self, pos, value):
            # print "Filtering with", pos, value
            IMP.domino.SubsetFilter.__init__(
                self, "MF" + str(pos) + " " + str(value))
            self.pos = pos
            self.value = value

        def get_next_state(self, pos, s):
            # suggest that the sampler try the correct state
            # this method is only called if the filter failed, so pos must be
            # self.pos
            if s[self.pos] > self.value:
                # a very large number
                return 2 ** 29
            else:
                return self.value

        def get_is_ok(self, state):
            # it is only OK if it has the required state
            ret = state[self.pos] == self.value
            return ret

    def get_strength(self, s, excluded):
        # return the maximum value since it dictates the state for the particle
        return 1

    def __init__(self, p, s):
        # set the name of the object to something vaguely useful
        IMP.domino.SubsetFilterTable.__init__(
            self, "MFT" + p.get_name() + " at " + str(s))
        self.p = p
        self.s = s

    def get_subset_filter(self, subset, excluded):
        # create a filter if self.p is in subset but not in excluded
        if self.p in subset and self.p not in sum([list(x) for x in excluded], []):
            # pass the position of self.p and the value that it must have
            return self.MyFilter(list(subset).index(self.p), self.s)
        else:
            return None


def create_sampler(m, ps, rs, pst):
    s = IMP.domino.DominoSampler(m, pst)
    s.set_restraints(rs)
    # s.set_log_level(IMP.VERBOSE)
    # the following lines recreate the defaults and so are optional
    filters = []
    # do not allow particles with the same ParticleStates object
    # to have the same state index
    filters.append(IMP.domino.ExclusionSubsetFilterTable(pst))
    rc = IMP.domino.RestraintCache(pst)
    rc.add_restraints(rs)
    # filter states that score worse than the cutoffs in the Model
    filters.append(IMP.domino.RestraintScoreSubsetFilterTable(rc))
    filters[-1].set_log_level(IMP.SILENT)
    mf = MyFilterTable(m.get_particle(ps[1]), 0)
    # try with and without this line
    filters.append(mf)
    states = IMP.domino.BranchAndBoundAssignmentsTable(pst, filters)
    # states.set_log_level(IMP.SILENT);
    s.set_assignments_table(states)
    s.set_subset_filter_tables(filters)
    return s

IMP.set_log_level(IMP.TERSE)
m = IMP.Model()
m.set_log_level(IMP.SILENT)

print("creating representation")
ps = create_representation(m)
print("creating discrete states")
pst = create_discrete_states(m, ps)
print("creating score function")
rs = create_scoring(m, ps)

print("creating sampler")
s = create_sampler(m, ps, rs, pst)

# s.set_log_level(IMP.SILENT)
print("sampling")
cs = s.create_sample()

print("found ", cs.get_number_of_configurations(), "solutions")
sf = IMP.core.RestraintsScoringFunction(rs)
for i in range(cs.get_number_of_configurations()):
    cs.load_configuration(i)
    print("solution number:", i, " is:", sf.evaluate(False))
    for p in ps:
        print(IMP.core.XYZ(m, p).get_x(), IMP.core.XYZ(m, p).get_y())



#!/usr/bin/env python

"""
Check for problems that might break the build in non-IMP repository code:
- a module without a README.md
- missing or incomplete submodules
"""

import os
import sys
import os.path
import tools


def check_readme():
    for module, g in tools.get_modules("."):
        if not os.path.exists(os.path.join(g, "README.md")):
            sys.stderr.write("Each module must have a file 'README.md' "
                             "describing what what it does and "
                             "who wrote it.\n")
            sys.stderr.write(module, "does not.\n")
            exit(1)


def main():
    check_readme()


if __name__ == '__main__':
    main()



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

# This wrapper is part of IMP,
# Copyright 2007-2021 IMP Inventors. All rights reserved.

from __future__ import print_function, division, absolute_import
from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

import _IMP_modeller

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


import weakref

class IMP_MODELLER_SwigPyIterator(object):
    r"""Proxy of C++ swig::IMP_MODELLER_SwigPyIterator class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _IMP_modeller.delete_IMP_MODELLER_SwigPyIterator

    def value(self):
        r"""value(IMP_MODELLER_SwigPyIterator self) -> PyObject *"""
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator_value(self)

    def incr(self, n=1):
        r"""incr(IMP_MODELLER_SwigPyIterator self, size_t n=1) -> IMP_MODELLER_SwigPyIterator"""
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator_incr(self, n)

    def decr(self, n=1):
        r"""decr(IMP_MODELLER_SwigPyIterator self, size_t n=1) -> IMP_MODELLER_SwigPyIterator"""
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator_decr(self, n)

    def distance(self, x):
        r"""distance(IMP_MODELLER_SwigPyIterator self, IMP_MODELLER_SwigPyIterator x) -> ptrdiff_t"""
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator_distance(self, x)

    def equal(self, x):
        r"""equal(IMP_MODELLER_SwigPyIterator self, IMP_MODELLER_SwigPyIterator x) -> bool"""
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator_equal(self, x)

    def copy(self):
        r"""copy(IMP_MODELLER_SwigPyIterator self) -> IMP_MODELLER_SwigPyIterator"""
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator_copy(self)

    def next(self):
        r"""next(IMP_MODELLER_SwigPyIterator self) -> PyObject *"""
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator_next(self)

    def __next__(self):
        r"""__next__(IMP_MODELLER_SwigPyIterator self) -> PyObject *"""
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator___next__(self)

    def previous(self):
        r"""previous(IMP_MODELLER_SwigPyIterator self) -> PyObject *"""
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator_previous(self)

    def advance(self, n):
        r"""advance(IMP_MODELLER_SwigPyIterator self, ptrdiff_t n) -> IMP_MODELLER_SwigPyIterator"""
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator_advance(self, n)

    def __eq__(self, x):
        r"""__eq__(IMP_MODELLER_SwigPyIterator self, IMP_MODELLER_SwigPyIterator x) -> bool"""
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator___eq__(self, x)

    def __ne__(self, x):
        r"""__ne__(IMP_MODELLER_SwigPyIterator self, IMP_MODELLER_SwigPyIterator x) -> bool"""
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator___ne__(self, x)

    def __iadd__(self, n):
        r"""__iadd__(IMP_MODELLER_SwigPyIterator self, ptrdiff_t n) -> IMP_MODELLER_SwigPyIterator"""
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator___iadd__(self, n)

    def __isub__(self, n):
        r"""__isub__(IMP_MODELLER_SwigPyIterator self, ptrdiff_t n) -> IMP_MODELLER_SwigPyIterator"""
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator___isub__(self, n)

    def __add__(self, n):
        r"""__add__(IMP_MODELLER_SwigPyIterator self, ptrdiff_t n) -> IMP_MODELLER_SwigPyIterator"""
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator___add__(self, n)

    def __sub__(self, *args):
        r"""
        __sub__(IMP_MODELLER_SwigPyIterator self, ptrdiff_t n) -> IMP_MODELLER_SwigPyIterator
        __sub__(IMP_MODELLER_SwigPyIterator self, IMP_MODELLER_SwigPyIterator x) -> ptrdiff_t
        """
        return _IMP_modeller.IMP_MODELLER_SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register IMP_MODELLER_SwigPyIterator in _IMP_modeller:
_IMP_modeller.IMP_MODELLER_SwigPyIterator_swigregister(IMP_MODELLER_SwigPyIterator)


_value_types=[]
_object_types=[]
_raii_types=[]
_plural_types=[]

IMP_DEBUG = _IMP_modeller.IMP_DEBUG

IMP_RELEASE = _IMP_modeller.IMP_RELEASE

IMP_SILENT = _IMP_modeller.IMP_SILENT

IMP_PROGRESS = _IMP_modeller.IMP_PROGRESS

IMP_TERSE = _IMP_modeller.IMP_TERSE

IMP_VERBOSE = _IMP_modeller.IMP_VERBOSE

IMP_MEMORY = _IMP_modeller.IMP_MEMORY

IMP_NONE = _IMP_modeller.IMP_NONE

IMP_USAGE = _IMP_modeller.IMP_USAGE

IMP_INTERNAL = _IMP_modeller.IMP_INTERNAL

IMP_KERNEL_HAS_LOG4CXX = _IMP_modeller.IMP_KERNEL_HAS_LOG4CXX

IMP_COMPILER_HAS_AUTO = _IMP_modeller.IMP_COMPILER_HAS_AUTO

IMP_COMPILER_HAS_DEBUG_VECTOR = _IMP_modeller.IMP_COMPILER_HAS_DEBUG_VECTOR

IMP_COMPILER_HAS_UNIQUE_PTR = _IMP_modeller.IMP_COMPILER_HAS_UNIQUE_PTR

IMP_KERNEL_HAS_BOOST_RANDOM = _IMP_modeller.IMP_KERNEL_HAS_BOOST_RANDOM

IMP_KERNEL_HAS_NUMPY = _IMP_modeller.IMP_KERNEL_HAS_NUMPY

IMP_KERNEL_HAS_GPERFTOOLS = _IMP_modeller.IMP_KERNEL_HAS_GPERFTOOLS

IMP_KERNEL_HAS_TCMALLOC_HEAPCHECKER = _IMP_modeller.IMP_KERNEL_HAS_TCMALLOC_HEAPCHECKER

IMP_KERNEL_HAS_TCMALLOC_HEAPPROFILER = _IMP_modeller.IMP_KERNEL_HAS_TCMALLOC_HEAPPROFILER

IMPKERNEL_SHOW_WARNINGS = _IMP_modeller.IMPKERNEL_SHOW_WARNINGS


import sys
class _DirectorObjects(object):
    """@internal Simple class to keep references to director objects
       to prevent premature deletion."""
    def __init__(self):
        self._objects = []
    def register(self, obj):
        """Take a reference to a director object; will only work for
           refcounted C++ classes"""
        if hasattr(obj, 'get_ref_count'):
            self._objects.append(obj)
    def cleanup(self):
        """Only drop our reference and allow cleanup by Python if no other
           Python references exist (we hold 3 references: one in self._objects,
           one in x, and one in the argument list for getrefcount) *and* no
           other C++ references exist (the Python object always holds one)"""
        objs = [x for x in self._objects if sys.getrefcount(x) > 3 \
                                         or x.get_ref_count() > 1]
# Do in two steps so the references are kept until the end of the
# function (deleting references may trigger a fresh call to this method)
        self._objects = objs
    def get_object_count(self):
        """Get number of director objects (useful for testing only)"""
        return len(self._objects)
_director_objects = _DirectorObjects()

class _ostream(object):
    r"""Proxy of C++ std::ostream class."""

    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined")
    __repr__ = _swig_repr

    def write(self, osa_buf):
        r"""write(_ostream self, char const * osa_buf)"""
        return _IMP_modeller._ostream_write(self, osa_buf)

# Register _ostream in _IMP_modeller:
_IMP_modeller._ostream_swigregister(_ostream)

IMP_COMPILER_HAS_OVERRIDE = _IMP_modeller.IMP_COMPILER_HAS_OVERRIDE

IMP_COMPILER_HAS_FINAL = _IMP_modeller.IMP_COMPILER_HAS_FINAL

IMP_HAS_NOEXCEPT = _IMP_modeller.IMP_HAS_NOEXCEPT

IMP_C_OPEN_BINARY = _IMP_modeller.IMP_C_OPEN_BINARY

import IMP
IMP_CGAL_HAS_BOOST_FILESYSTEM = _IMP_modeller.IMP_CGAL_HAS_BOOST_FILESYSTEM

IMP_CGAL_HAS_BOOST_PROGRAMOPTIONS = _IMP_modeller.IMP_CGAL_HAS_BOOST_PROGRAMOPTIONS

IMP_CGAL_HAS_BOOST_RANDOM = _IMP_modeller.IMP_CGAL_HAS_BOOST_RANDOM

IMP_CGAL_HAS_BOOST_SYSTEM = _IMP_modeller.IMP_CGAL_HAS_BOOST_SYSTEM

IMP_CGAL_HAS_NUMPY = _IMP_modeller.IMP_CGAL_HAS_NUMPY

IMPCGAL_SHOW_WARNINGS = _IMP_modeller.IMPCGAL_SHOW_WARNINGS

import IMP.cgal
IMP_ALGEBRA_HAS_IMP_CGAL = _IMP_modeller.IMP_ALGEBRA_HAS_IMP_CGAL

IMP_ALGEBRA_HAS_BOOST_FILESYSTEM = _IMP_modeller.IMP_ALGEBRA_HAS_BOOST_FILESYSTEM

IMP_ALGEBRA_HAS_BOOST_PROGRAMOPTIONS = _IMP_modeller.IMP_ALGEBRA_HAS_BOOST_PROGRAMOPTIONS

IMP_ALGEBRA_HAS_BOOST_RANDOM = _IMP_modeller.IMP_ALGEBRA_HAS_BOOST_RANDOM

IMP_ALGEBRA_HAS_BOOST_SYSTEM = _IMP_modeller.IMP_ALGEBRA_HAS_BOOST_SYSTEM

IMP_ALGEBRA_HAS_CGAL = _IMP_modeller.IMP_ALGEBRA_HAS_CGAL

IMP_ALGEBRA_HAS_NUMPY = _IMP_modeller.IMP_ALGEBRA_HAS_NUMPY

IMP_ALGEBRA_HAS_ANN = _IMP_modeller.IMP_ALGEBRA_HAS_ANN

IMPALGEBRA_SHOW_WARNINGS = _IMP_modeller.IMPALGEBRA_SHOW_WARNINGS

import IMP.algebra
IMP_DISPLAY_HAS_IMP_CGAL = _IMP_modeller.IMP_DISPLAY_HAS_IMP_CGAL

IMP_DISPLAY_HAS_BOOST_FILESYSTEM = _IMP_modeller.IMP_DISPLAY_HAS_BOOST_FILESYSTEM

IMP_DISPLAY_HAS_BOOST_PROGRAMOPTIONS = _IMP_modeller.IMP_DISPLAY_HAS_BOOST_PROGRAMOPTIONS

IMP_DISPLAY_HAS_BOOST_RANDOM = _IMP_modeller.IMP_DISPLAY_HAS_BOOST_RANDOM

IMP_DISPLAY_HAS_BOOST_SYSTEM = _IMP_modeller.IMP_DISPLAY_HAS_BOOST_SYSTEM

IMP_DISPLAY_HAS_CGAL = _IMP_modeller.IMP_DISPLAY_HAS_CGAL

IMP_DISPLAY_HAS_NUMPY = _IMP_modeller.IMP_DISPLAY_HAS_NUMPY

IMPDISPLAY_SHOW_WARNINGS = _IMP_modeller.IMPDISPLAY_SHOW_WARNINGS

import IMP.display
IMP_SCORE_FUNCTOR_HAS_IMP_CGAL = _IMP_modeller.IMP_SCORE_FUNCTOR_HAS_IMP_CGAL

IMP_SCORE_FUNCTOR_HAS_BOOST_FILESYSTEM = _IMP_modeller.IMP_SCORE_FUNCTOR_HAS_BOOST_FILESYSTEM

IMP_SCORE_FUNCTOR_HAS_BOOST_PROGRAMOPTIONS = _IMP_modeller.IMP_SCORE_FUNCTOR_HAS_BOOST_PROGRAMOPTIONS

IMP_SCORE_FUNCTOR_HAS_BOOST_RANDOM = _IMP_modeller.IMP_SCORE_FUNCTOR_HAS_BOOST_RANDOM

IMP_SCORE_FUNCTOR_HAS_BOOST_SYSTEM = _IMP_modeller.IMP_SCORE_FUNCTOR_HAS_BOOST_SYSTEM

IMP_SCORE_FUNCTOR_HAS_CGAL = _IMP_modeller.IMP_SCORE_FUNCTOR_HAS_CGAL

IMP_SCORE_FUNCTOR_HAS_HDF5 = _IMP_modeller.IMP_SCORE_FUNCTOR_HAS_HDF5

IMP_SCORE_FUNCTOR_HAS_NUMPY = _IMP_modeller.IMP_SCORE_FUNCTOR_HAS_NUMPY

IMPSCOREFUNCTOR_SHOW_WARNINGS = _IMP_modeller.IMPSCOREFUNCTOR_SHOW_WARNINGS

import IMP.score_functor
IMP_CORE_HAS_IMP_CGAL = _IMP_modeller.IMP_CORE_HAS_IMP_CGAL

IMP_CORE_HAS_IMP_KERNEL = _IMP_modeller.IMP_CORE_HAS_IMP_KERNEL

IMP_CORE_HAS_BOOST_FILESYSTEM = _IMP_modeller.IMP_CORE_HAS_BOOST_FILESYSTEM

IMP_CORE_HAS_BOOST_PROGRAMOPTIONS = _IMP_modeller.IMP_CORE_HAS_BOOST_PROGRAMOPTIONS

IMP_CORE_HAS_BOOST_RANDOM = _IMP_modeller.IMP_CORE_HAS_BOOST_RANDOM

IMP_CORE_HAS_BOOST_SYSTEM = _IMP_modeller.IMP_CORE_HAS_BOOST_SYSTEM

IMP_CORE_HAS_CGAL = _IMP_modeller.IMP_CORE_HAS_CGAL

IMP_CORE_HAS_HDF5 = _IMP_modeller.IMP_CORE_HAS_HDF5

IMP_CORE_HAS_NUMPY = _IMP_modeller.IMP_CORE_HAS_NUMPY

IMPCORE_SHOW_WARNINGS = _IMP_modeller.IMPCORE_SHOW_WARNINGS

import IMP.core
IMP_MODELLER_HAS_IMP_ALGEBRA = _IMP_modeller.IMP_MODELLER_HAS_IMP_ALGEBRA

IMP_MODELLER_HAS_IMP_CGAL = _IMP_modeller.IMP_MODELLER_HAS_IMP_CGAL

IMP_MODELLER_HAS_IMP_DISPLAY = _IMP_modeller.IMP_MODELLER_HAS_IMP_DISPLAY

IMP_MODELLER_HAS_IMP_KERNEL = _IMP_modeller.IMP_MODELLER_HAS_IMP_KERNEL

IMP_MODELLER_HAS_IMP_SCORE_FUNCTOR = _IMP_modeller.IMP_MODELLER_HAS_IMP_SCORE_FUNCTOR

IMP_MODELLER_HAS_BOOST_FILESYSTEM = _IMP_modeller.IMP_MODELLER_HAS_BOOST_FILESYSTEM

IMP_MODELLER_HAS_BOOST_PROGRAMOPTIONS = _IMP_modeller.IMP_MODELLER_HAS_BOOST_PROGRAMOPTIONS

IMP_MODELLER_HAS_BOOST_RANDOM = _IMP_modeller.IMP_MODELLER_HAS_BOOST_RANDOM

IMP_MODELLER_HAS_BOOST_SYSTEM = _IMP_modeller.IMP_MODELLER_HAS_BOOST_SYSTEM

IMP_MODELLER_HAS_CGAL = _IMP_modeller.IMP_MODELLER_HAS_CGAL

IMP_MODELLER_HAS_HDF5 = _IMP_modeller.IMP_MODELLER_HAS_HDF5

IMP_MODELLER_HAS_NUMPY = _IMP_modeller.IMP_MODELLER_HAS_NUMPY

IMP_MODELLER_HAS_PYTHON_IHM = _IMP_modeller.IMP_MODELLER_HAS_PYTHON_IHM

IMPMODELLER_SHOW_WARNINGS = _IMP_modeller.IMPMODELLER_SHOW_WARNINGS



import math
import imp
import os
import sys
import tempfile
import shutil
import IMP.atom
import IMP.container
import modeller.scripts
import modeller.optimizers


# Use Modeller 10 class names
if not hasattr(modeller.terms, 'EnergyTerm'):
    modeller.terms.EnergyTerm = modeller.terms.energy_term
    modeller.Selection = modeller.selection


class _TempDir(object):
    """Make a temporary directory that is deleted when the object is."""

    def __init__(self):
        self.tmpdir = tempfile.mkdtemp()

    def __del__(self):
        shutil.rmtree(self.tmpdir, ignore_errors=True)


class IMPRestraints(modeller.terms.EnergyTerm):
    """A Modeller restraint which evaluates an IMP scoring function.
       This can be used to incorporate IMP Restraints into an existing
       comparative modeling pipeline, or to use Modeller optimizers or
       protocols.
    """

    _physical_type = modeller.physical.absposition

    def __init__(self, particles, scoring_function=None):
        """Constructor.
           @param particles A list of the IMP atoms (as Particle objects),
                            same order as the Modeller atoms.
           @param scoring_function An IMP::ScoringFunction object that will
                            be incorporated into the Modeller score (molpdf).
           @note since Modeller, unlike IMP, is sensitive to the ordering
                 of atoms, it usually makes sense to create the model in
                 Modeller and then use ModelLoader to load it into IMP,
                 since that will preserve the Modeller atom ordering in IMP.
        """
        modeller.terms.EnergyTerm.__init__(self)
        self._particles = particles
        if scoring_function:
            self._sf = scoring_function
        else:
            self._sf = particles[0].get_model()

    def eval(self, mdl, deriv, indats):
        atoms = self.indices_to_atoms(mdl, indats)
        _copy_modeller_coords_to_imp(atoms, self._particles)
        if len(self._particles) == 0:
            score = 0.
        else:
            score = self._sf.evaluate(deriv)
        if deriv:
            dvx = [0.] * len(indats)
            dvy = [0.] * len(indats)
            dvz = [0.] * len(indats)
            _get_imp_derivs(self._particles, dvx, dvy, dvz)
            return (score, dvx, dvy, dvz)
        else:
            return score


class ModellerRestraints(IMP.Restraint):
    """An IMP restraint using all defined Modeller restraints.
       This is useful if you want to use Modeller restraints with an IMP
       optimizer, or in combination with IMP restraints.

       @note Currently only the coordinates of the atoms are translated
             between Modeller and IMP; thus, a Modeller restraint which
             uses any other attribute (e.g. charge) will not react if
             this attribute is changed by IMP.
    """

    def __init__(self, model, modeller_model, particles):
        """Constructor.
           @param model The IMP Model object.
           @param modeller_model The Modeller model object.
           @param particles A list of the IMP atoms (as Particle objects),
                            in the same order as the Modeller atoms.
           @note since Modeller, unlike IMP, is sensitive to the ordering
                 of atoms, it usually makes sense to create the model in
                 Modeller and then use ModelLoader to load it into IMP,
                 since that will preserve the Modeller atom ordering in IMP.
        """
        def get_particle(x):
            if hasattr(x, 'get_particle'):
                return x.get_particle()
            else:
                return x
        IMP.Restraint.__init__(self, model, "ModellerRestraints %1%")
        self._modeller_model = modeller_model
        self._particles = [get_particle(x) for x in particles]

    def unprotected_evaluate(self, accum):
        atoms = self._modeller_model.atoms
        sel = modeller.Selection(self._modeller_model)
        _copy_imp_coords_to_modeller(self._particles, atoms)
        energies = sel.energy()
        if accum:
            _add_modeller_derivs_to_imp(atoms, self._particles, accum)

        return energies[0]

    def get_version_info(self):
        return IMP.VersionInfo("IMP developers", "0.1")
    def do_show(self, fh):
        fh.write("ModellerRestraints")
    def do_get_inputs(self):
        return self._particles


def _copy_imp_coords_to_modeller(particles, atoms):
    """Copy atom coordinates from IMP to Modeller"""
    xkey = IMP.FloatKey("x")
    ykey = IMP.FloatKey("y")
    zkey = IMP.FloatKey("z")
    for (num, at) in enumerate(atoms):
        at.x = particles[num].get_value(xkey)
        at.y = particles[num].get_value(ykey)
        at.z = particles[num].get_value(zkey)


def _copy_modeller_coords_to_imp(atoms, particles):
    """Copy atom coordinates from Modeller to IMP"""
    xkey = IMP.FloatKey("x")
    ykey = IMP.FloatKey("y")
    zkey = IMP.FloatKey("z")
    for (num, at) in enumerate(atoms):
        particles[num].set_value(xkey, at.x)
        particles[num].set_value(ykey, at.y)
        particles[num].set_value(zkey, at.z)


def _add_modeller_derivs_to_imp(atoms, particles, accum):
    """Add atom derivatives from Modeller to IMP"""
    for (num, at) in enumerate(atoms):
        xyz = IMP.core.XYZ(particles[num])
        xyz.add_to_derivative(0, at.dvx, accum)
        xyz.add_to_derivative(1, at.dvy, accum)
        xyz.add_to_derivative(2, at.dvz, accum)


def _get_imp_derivs(particles, dvx, dvy, dvz):
    """Move atom derivatives from IMP to Modeller"""
    xkey = IMP.FloatKey("x")
    ykey = IMP.FloatKey("y")
    zkey = IMP.FloatKey("z")
    for idx in range(0, len(dvx)):
        dvx[idx] = particles[idx].get_derivative(xkey)
        dvy[idx] = particles[idx].get_derivative(ykey)
        dvz[idx] = particles[idx].get_derivative(zkey)


# Generators to create IMP UnaryFunction objects from Modeller parameters:
def _HarmonicLowerBoundGenerator(parameters, modalities):
    (mean, stdev) = parameters
    k = IMP.core.Harmonic.get_k_from_standard_deviation(stdev)
    return IMP.core.HarmonicLowerBound(mean, k)

def _HarmonicUpperBoundGenerator(parameters, modalities):
    (mean, stdev) = parameters
    k = IMP.core.Harmonic.get_k_from_standard_deviation(stdev)
    return IMP.core.HarmonicUpperBound(mean, k)

def _HarmonicGenerator(parameters, modalities):
    (mean, stdev) = parameters
    k = IMP.core.Harmonic.get_k_from_standard_deviation(stdev)
    return IMP.core.Harmonic(mean, k)

def _CosineGenerator(parameters, modalities):
    (phase, force_constant) = parameters
    (periodicity,) = modalities
    return IMP.core.Cosine(force_constant, periodicity, phase)

def _LinearGenerator(parameters, modalities):
    (scale,) = parameters
    return IMP.core.Linear(0, scale)

def _SplineGenerator(parameters, modalities):
    (open, low, high, delta, lowderiv, highderiv) = parameters[:6]
    values = []
    for v in parameters[6:]:
        values.append(v)
    if open < 0.0:
        return IMP.core.ClosedCubicSpline(values, low, delta)
    else:
        return IMP.core.OpenCubicSpline(values, low, delta)

#: Mapping from Modeller math form number to a unary function generator
_unary_func_generators = {
    1: _HarmonicLowerBoundGenerator,
    2: _HarmonicUpperBoundGenerator,
    3: _HarmonicGenerator,
    7: _CosineGenerator,
    8: _LinearGenerator,
    10: _SplineGenerator,
}

# Generators to make IMP Restraint objects from Modeller features
def _DistanceRestraintGenerator(form, modalities, atoms, parameters):
    unary_func_gen = _unary_func_generators[form]
    return IMP.core.DistanceRestraint(atoms[0].get_model(),
                                      unary_func_gen(parameters, modalities),
                                      atoms[0], atoms[1])

def _AngleRestraintGenerator(form, modalities, atoms, parameters):
    unary_func_gen = _unary_func_generators[form]
    return IMP.core.AngleRestraint(atoms[0].get_model(),
                                   unary_func_gen(parameters, modalities),
                                   atoms[0], atoms[1], atoms[2])

def _MultiBinormalGenerator(form, modalities, atoms, parameters):
    nterms = modalities[0]
    if len(parameters) != nterms * 6:
        raise ValueError("Incorrect number of parameters (%d) for multiple "
                         "binormal restraint - expecting %d (%d terms * 6)" \
                         % (len(parameters), nterms * 6, nterms))
    r = IMP.core.MultipleBinormalRestraint(atoms[0].get_model(),
                                           atoms[:4], atoms[4:8])
    for i in range(nterms):
        t = IMP.core.BinormalTerm()
        t.set_weight(parameters[i])
        t.set_means((parameters[nterms + i * 2],
                     parameters[nterms + i * 2 + 1]))
        t.set_standard_deviations((parameters[nterms * 3 + i * 2],
                                   parameters[nterms * 3 + i * 2 + 1]))
        t.set_correlation(parameters[nterms * 5 + i])
        r.add_term(t)
    return r

def _DihedralRestraintGenerator(form, modalities, atoms, parameters):
    if form == 9:
        return _MultiBinormalGenerator(form, modalities, atoms, parameters)
    unary_func_gen = _unary_func_generators[form]
    return IMP.core.DihedralRestraint(atoms[0].get_model(),
                                      unary_func_gen(parameters, modalities),
                                      atoms[0], atoms[1], atoms[2], atoms[3])

def _get_protein_atom_particles(protein):
    """Given a protein particle, get the flattened list of all child atoms"""
    atom_particles = []
    for ichain in range(protein.get_number_of_children()):
        chain = protein.get_child(ichain)
        for ires in range(chain.get_number_of_children()):
            residue = chain.get_child(ires)
            for iatom in range(residue.get_number_of_children()):
                atom = residue.get_child(iatom)
                atom_particles.append(atom.get_particle())
    return atom_particles

def _load_restraints_line(line, atom_particles):
    """Parse a single Modeller restraints file line and return the
       corresponding IMP restraint."""
    spl = line.split()
    typ = spl.pop(0)
    if typ == 'MODELLER5':
        return
    elif typ != 'R':
        raise NotImplementedError("Only 'R' lines currently read from " + \
                                  "Modeller restraints files")
    form = int(spl.pop(0))
    modalities = [int(spl.pop(0))]
    features = [int(spl.pop(0))]
# Discard group
    spl.pop(0)
    natoms = [int(spl.pop(0))]
    nparam = int(spl.pop(0))
    nfeat = int(spl.pop(0))
    for i in range(nfeat - 1):
        modalities.append(int(spl.pop(0)))
        features.append(int(spl.pop(0)))
        natoms.append(int(spl.pop(0)))
    atoms = [int(spl.pop(0)) for x in range(natoms[0])]
    for i in range(len(atoms)):
        atoms[i] = atom_particles[atoms[i] - 1]
    parameters = [float(spl.pop(0)) for x in range(nparam)]
    restraint_generators = {
        1 : _DistanceRestraintGenerator,
        2 : _AngleRestraintGenerator,
        3 : _DihedralRestraintGenerator,
        4 : _DihedralRestraintGenerator,
    }
    restraint_gen = restraint_generators[features[0]]
    return restraint_gen(form, modalities, atoms, parameters)


def _load_entire_restraints_file(filename, protein):
    """Yield a set of IMP restraints from a Modeller restraints file."""
    atoms = _get_protein_atom_particles(protein)
    with open(filename, 'r') as fh:
        for line in fh:
            try:
                rsr = _load_restraints_line(line, atoms)
                if rsr is not None:
                    yield rsr
            except Exception as err:
                print("Cannot read restraints file line:\n" + line)
                raise


def _copy_residue(r, model):
    """Copy residue information from modeller to imp"""
#print "residue "+str(r)
    p=IMP.Particle(model)
    rp= IMP.atom.Residue.setup_particle(p, IMP.atom.ResidueType(r.pdb_name),
                                         r.index)
    p.set_name(str("residue "+r.num));
    return p


def _copy_atom(a, model):
    """Copy atom information from modeller"""
#print "atom "+str(a)
    p=IMP.Particle(model)
    ap= IMP.atom.Atom.setup_particle(p,IMP.atom.AtomType(a.name))
    xyzd= IMP.core.XYZ.setup_particle(p, IMP.algebra.Vector3D(a.x, a.y, a.z))
# Alignment structures don't have charges or atom types; models do
    if hasattr(a, 'charge'):
        IMP.atom.Charged.setup_particle(p, a.charge)
    if hasattr(a, 'type'):
        IMP.atom.CHARMMAtom.setup_particle(p, a.type.name)
    ap.set_input_index(a.index)
    return p

def _copy_chain(c, model):
    """Copy chain information from modeller"""
#print "atom "+str(a)
    p=IMP.Particle(model)
#set the chain name
    cp = IMP.atom.Chain.setup_particle(p,c.name)
    return p

def _get_forcefield(submodel):
    if submodel == 3:
        ff = IMP.atom.CHARMMParameters(
                 IMP.atom.get_data_path('top_heav.lib'),
                 IMP.atom.get_data_path('par.lib'))
    else:
        ff = IMP.atom.CHARMMParameters(
                 IMP.atom.get_data_path('top.lib'),
                 IMP.atom.get_data_path('par.lib'))
    return ff

def add_soft_sphere_radii(hierarchy, submodel, scale=1.0, filename=None):
    """Add radii to the hierarchy using the Modeller radius library, radii.lib.
       Each radius is scaled by the given scale (Modeller usually scales radii
       by a factor of 0.82). submodel specifies the topology submodel, which is
       the column in radii.lib to use."""
    if filename is None:
        filename = IMP.atom.get_data_path('radii.lib')
    radii = {}
    with open(filename) as fh:
        for line in fh:
            if line.startswith('#'): continue
            spl = line.split()
            if len(spl) > 11:
                radii[spl[0]] = float(spl[submodel])
    atoms = IMP.atom.get_by_type(hierarchy, IMP.atom.ATOM_TYPE)
    for a in atoms:
        p = a.get_particle()
        ct = IMP.atom.CHARMMAtom(p).get_charmm_type()
        if ct in radii:
            radius = radii[ct] * scale
            if IMP.core.XYZR.get_is_setup(p):
                IMP.core.XYZR(p).set_radius(radius)
            else:
                IMP.core.XYZR.setup_particle(p, radius)


class ModelLoader(object):
    """Read a Modeller model into IMP. After creating this object, the atoms
       in the Modeller model can be loaded into IMP using the load_atoms()
       method, then optionally any Modeller static restraints can be read in
       with load_static_restraints() or load_static_restraints_file().

       This class can also be used to read Modeller alignment structures;
       however, only load_atoms() will be useful in such a case (since
       alignment structures don't have restraints or other information).

    """

    def __init__(self, modeller_model):
        """Constructor.
           @param modeller_model The Modeller model or alignment structure
                                 object to read.
        """
        self._modeller_model = modeller_model

    def load_atoms(self, model):
        """Construct an IMP::atom::Hierarchy that contains the same atoms as
           the Modeller model or alignment structure.

           IMP atoms created from a Modeller model will be given charges and
           CHARMM types, extracted from the model. Alignment structures don't
           contain this information, so the IMP atoms won't either.

           @param model The IMP::Model object in which the hierarchy will be
                        created. The highest level hierarchy node is a PROTEIN.
           @return the newly-created root IMP::atom::Hierarchy.
        """
        pp = IMP.Particle(model)
        hpp = IMP.atom.Hierarchy.setup_particle(pp)
        self._atoms = {}
        for chain in self._modeller_model.chains:
            cp = IMP.Particle(model)
            hcp = IMP.atom.Chain.setup_particle(cp, chain.name)
# We don't really know the type yet
            hpp.add_child(hcp)
            for residue in chain.residues:
                rp = _copy_residue(residue, model)
                hrp = IMP.atom.Hierarchy(rp)
                hcp.add_child(hrp)
                for atom in residue.atoms:
                    ap = _copy_atom(atom, model)
                    hap = IMP.atom.Hierarchy(ap)
                    hrp.add_child(hap)
                    self._atoms[atom.index] = ap
                lastres = hrp
        self._modeller_hierarchy = hpp
        return hpp

    def _get_nonbonded_list(self, atoms, pair_filter, edat, distance):
        nbl = IMP.container.ClosePairContainer(atoms, distance,
                                               edat.update_dynamic)

# Exclude the same sets of atoms as Modeller
        if pair_filter is None:
            pair_filter = IMP.atom.StereochemistryPairFilter()
            if edat.excl_local[0]:
                pair_filter.set_bonds(list(self.load_bonds()))
            if edat.excl_local[1]:
                pair_filter.set_angles(list(self.load_angles()))
            if edat.excl_local[2]:
                pair_filter.set_dihedrals(list(self.load_dihedrals()))
        nbl.add_pair_filter(pair_filter)
        return nbl

    def load_bonds(self):
        """Load the Modeller bond topology into the IMP model. Each bond is
           represented in IMP as an IMP::atom::Bond, with no defined length
           or stiffness. These bonds are primarily useful as input to
           IMP::atom::StereochemistryPairFilter, to exclude bond interactions
           from the nonbonded list. Typically the contribution to the scoring
           function from the bonds is included in the Modeller static restraints
           (use load_static_restraints() or load_static_restraints_file() to
           load these). If you want to regenerate the stereochemistry in IMP,
           do not use these functions (as then stereochemistry scoring terms
           and exclusions would be double-counted) and instead use the
           IMP::atom::CHARMMTopology class.

           You must call load_atoms() prior to using this function.
           @see load_angles(), load_dihedrals(), load_impropers()
           @return A generator listing all of the bonds.
        """
        if not hasattr(self, '_modeller_hierarchy'):
            raise ValueError("Call load_atoms() first.")
        for (maa, mab) in self._modeller_model.bonds:
            pa = self._atoms[maa.index]
            pb = self._atoms[mab.index]
            if IMP.atom.Bonded.get_is_setup(pa):
                ba= IMP.atom.Bonded(pa)
            else:
                ba= IMP.atom.Bonded.setup_particle(pa)
            if IMP.atom.Bonded.get_is_setup(pb):
                bb= IMP.atom.Bonded(pb)
            else:
                bb= IMP.atom.Bonded.setup_particle(pb)
            yield IMP.atom.create_bond(ba, bb,
                                       IMP.atom.Bond.SINGLE).get_particle()

    def load_angles(self):
        """Load the Modeller angle topology into the IMP model.
           See load_bonds() for more details."""
        return self._internal_load_angles(self._modeller_model.angles,
                                          IMP.atom.Angle)

    def load_dihedrals(self):
        """Load the Modeller dihedral topology into the IMP model.
           See load_bonds() for more details."""
        return self._internal_load_angles(self._modeller_model.dihedrals,
                                          IMP.atom.Dihedral)

    def load_impropers(self):
        """Load the Modeller improper topology into the IMP model.
           See load_bonds() for more details."""
        return self._internal_load_angles(self._modeller_model.impropers,
                                          IMP.atom.Dihedral)

    def _internal_load_angles(self, angles, angle_class):
        if not hasattr(self, '_modeller_hierarchy'):
            raise ValueError("Call load_atoms() first.")
        for modeller_atoms in angles:
            imp_particles = [self._atoms[x.index] for x in modeller_atoms]
            p = IMP.Particle(imp_particles[0].get_model())
            a = angle_class.setup_particle(p,
                                 *[IMP.core.XYZ(x) for x in imp_particles])
            yield a.get_particle()

    def load_static_restraints_file(self, filename):
        """Convert a Modeller static restraints file into equivalent
           IMP::Restraints. load_atoms() must have been called first to read
           in the atoms that the restraints will act upon.
           @param filename Name of the Modeller restraints file. The restraints
                  in this file are assumed to act upon the model read in by
                  load_atoms(); no checking is done to enforce this.
           @return A Python generator of the newly-created IMP::Restraint
                   objects.
        """
        if not hasattr(self, '_modeller_hierarchy'):
            raise ValueError("Call load_atoms() first.")
        return _load_entire_restraints_file(filename, self._modeller_hierarchy)


    def load_static_restraints(self):
        """Convert the current set of Modeller static restraints into equivalent
           IMP::Restraints. load_atoms() must have been called first to read
           in the atoms that the restraints will act upon.
           @return A Python generator of the newly-created IMP::Restraint
                   objects.
        """
        class _RestraintGenerator(object):
            """Simple generator wrapper"""
            def __init__(self, gen):
                self._gen = gen
            def __iter__(self, *args, **keys):
                return self
            def close(self, *args, **keys):
                return self._gen.close(*args, **keys)
            def next(self):
                return next(self._gen)
            __next__ = next
            def send(self, *args, **keys):
                return self._gen.send(*args, **keys)
            def throw(self, *args, **keys):
                return self._gen.throw(*args, **keys)
# Write current restraints into a temporary file
        t = _TempDir()
        rsrfile = os.path.join(t.tmpdir, 'restraints.rsr')
        self._modeller_model.restraints.write(file=rsrfile)
# Read the file back in
        gen = self.load_static_restraints_file(rsrfile)
        wrap = _RestraintGenerator(gen)
# Ensure that tmpdir remains until we're done with the generator
        wrap._tempdir = t
        return wrap

    def load_dynamic_restraints(self, pair_filter=None):
        """Convert Modeller dynamic restraints into IMP::Restraint objects.

           For each currently active Modeller dynamic restraint
           (e.g. soft-sphere, electrostatics) an equivalent IMP::Restraint
           is created.
           load_atoms() must have been called first to read
           in the atoms that the restraints will act upon.

           If pair_filter is given, it is an IMP::PairFilter object to exclude
           pairs from the nonbonded lists used by the dynamic restraints.
           Otherwise, an IMP::atom::StereochemistryPairFilter object is created
           to exclude Modeller bonds, angles and dihedrals, as specified by
           edat.excl_local. (Note that this calls load_bonds(), load_angles()
           and load_dihedrals(), so will create duplicate lists of bonds if
           those methods are called manually as well.)

           @note Currently only soft-sphere, electrostatic and Lennard-Jones
                 restraints are loaded.
           @return A Python generator of the newly-created IMP::Restraint
                   objects.
        """
        if not hasattr(self, '_modeller_hierarchy'):
            raise ValueError("Call load_atoms() first.")
        edat = self._modeller_model.env.edat
        libs = self._modeller_model.env.libs
        atoms = IMP.atom.get_leaves(self._modeller_hierarchy)
        m = atoms[0].get_model()
        atoms = IMP.container.ListSingletonContainer(m, IMP.get_indexes(atoms))

        if edat.dynamic_sphere:
# Note: cannot use Modeller's cutoff distance, as that is
# center-to-center; IMP's is sphere surface-surface
            nbl = self._get_nonbonded_list(atoms, pair_filter, edat, 0.)
# No way to get Modeller radii, so we have to reassign them
            add_soft_sphere_radii(self._modeller_hierarchy,
                                  libs.topology.submodel, edat.radii_factor)
            k = \
              IMP.core.Harmonic.get_k_from_standard_deviation(edat.sphere_stdv)
            ps = IMP.core.SphereDistancePairScore(
                              IMP.core.HarmonicLowerBound(0, k))
            yield IMP.container.PairsRestraint(ps, nbl)

        if edat.dynamic_lennard or edat.dynamic_coulomb:
# 3.0 is roughly the max. atom diameter
            d = max(edat.contact_shell - 3.0, 0.0)
            nbl = self._get_nonbonded_list(atoms, pair_filter, edat, d)
            ff = _get_forcefield(libs.topology.submodel)
            ff.add_radii(self._modeller_hierarchy)

            if edat.dynamic_lennard:
                ff.add_well_depths(self._modeller_hierarchy)
                sf = IMP.atom.ForceSwitch(edat.lennard_jones_switch[0],
                                          edat.lennard_jones_switch[1])
                ps = IMP.atom.LennardJonesPairScore(sf)
                yield IMP.container.PairsRestraint(ps, nbl)

            if edat.dynamic_coulomb:
                sf = IMP.atom.ForceSwitch(edat.coulomb_switch[0],
                                          edat.coulomb_switch[1])
                ps = IMP.atom.CoulombPairScore(sf)
                ps.set_relative_dielectric(edat.relative_dielectric)
                yield IMP.container.PairsRestraint(ps, nbl)



def get_module_version():
    r"""get_module_version() -> std::string const"""
    return _IMP_modeller.get_module_version()

def get_example_path(fname):
    r"""get_example_path(std::string fname) -> std::string"""
    return _IMP_modeller.get_example_path(fname)

def get_data_path(fname):
    r"""get_data_path(std::string fname) -> std::string"""
    return _IMP_modeller.get_data_path(fname)

from . import _version_check
_version_check.check_version(get_module_version())
__version__ = get_module_version()



